@@ -1,4 +1,4 @@
-/* $Id: bogoconfig.c,v 1.192 2004/09/24 01:03:25 relson Exp $ */
+/* $Id: bogoconfig.c,v 1.215 2005/04/04 11:16:18 relson Exp $ */
 
 /*****************************************************************************
 
@@ -43,16 +43,17 @@ Note: bogolexer also uses configfile.c.
 
 #include "bogoconfig.h"
 #include "bogofilter.h"
-#include "bogohome.h"
 #include "bogoreader.h"
 #include "bool.h"
 #include "charset.h"
 #include "configfile.h"
 #include "datastore.h"
+#include "datastore_db.h"
 #include "error.h"
 #include "find_home.h"
 #include "format.h"
 #include "lexer.h"
+#include "longoptions.h"
 #include "maint.h"
 #include "paths.h"
 #include "score.h"
@@ -90,7 +91,14 @@ static void comma_parse(char opt, const char *arg, double *parm1, double *parm2,
 
 /*---------------------------------------------------------------------------*/
 
-struct option long_options[] = {
+static struct option longopts_bogofilter[] = {
+    /* longoptions.h - common options */
+    LONGOPTIONS_COMMON
+    /* longoptions.h - bogofilter/-lexer options */
+    LONGOPTIONS_LEX
+    /* longoptions.h - bogofilter/bogoutil options */
+    LONGOPTIONS_DB
+    /* bogofilter specific options */
     { "classify-files",			N, 0, 'B' },
     { "syslog-tag",			N, 0, 'L' },
     { "classify-mbox",			N, 0, 'M' },
@@ -99,55 +107,46 @@ struct option long_options[] = {
     { "unregister-spam",		N, 0, 'S' },
     { "fixed-terse-format",		N, 0, 'T' },
     { "report-unsure",			N, 0, 'U' },
-    { "version",			N, 0, 'V' },
     { "classify-stdin",			N, 0, 'b' },
-    { "bogofilter_dir",			R, 0, 'd' },
+    { "bogofilter-dir",			R, 0, 'd' },
     { "nonspam-exits-zero",		N, 0, 'e' },
-    { "help",				N, 0, 'h' },
-    { "db_cachesize",			N, 0, 'k' },
     { "use-syslog",			N, 0, 'l' },
     { "register-ham",			N, 0, 'n' },
     { "passthrough",			N, 0, 'p' },
     { "register-spam",			N, 0, 's' },
     { "update-as-scored",		N, 0, 'u' },
-    { "timestamp-date",			N, 0, 'y' },
-    { "config-file",			R, 0, 'c' },
-    { "no-config-file",			N, 0, 'C' },
     { "debug-flags",			R, 0, 'x' },
     { "debug-to-stdout",		N, 0, 'D' },
     { "no-header-tags",			N, 0, 'H' },
-    { "input-file",			N, 0, 'I' },
-    { "output-file",			N, 0, 'O' },
     { "query",				N, 0, 'Q' },
-    { "verbosity",			N, 0, 'v' },
-    { "block_on_subnets",		R, 0, O_BLOCK_ON_SUBNETS },
-    { "charset_default",		R, 0, O_CHARSET_DEFAULT },
-    { "ns_esf",				R, 0, O_NS_ESF },
-    { "sp_esf",				R, 0, O_SP_ESF },
-    { "ham_cutoff",			R, 0, O_HAM_CUTOFF },
-    { "header_format",			R, 0, O_HEADER_FORMAT },
-    { "log_header_format",		R, 0, O_LOG_HEADER_FORMAT },
-    { "log_update_format",		R, 0, O_LOG_UPDATE_FORMAT },
-    { "min_dev",			R, 0, O_MIN_DEV },
-    { "replace_nonascii_characters",	R, 0, O_REPLACE_NONASCII_CHARACTERS },
+    { "db-cachesize",			N, 0, 'k' },
+    { "ns-esf",				R, 0, O_NS_ESF },
+    { "sp-esf",				R, 0, O_SP_ESF },
+    { "ham-cutoff",			R, 0, O_HAM_CUTOFF },
+    { "header-format",			R, 0, O_HEADER_FORMAT },
+    { "log-header-format",		R, 0, O_LOG_HEADER_FORMAT },
+    { "log-update-format",		R, 0, O_LOG_UPDATE_FORMAT },
+    { "min-dev",			R, 0, O_MIN_DEV },
     { "robs",				R, 0, O_ROBS },
     { "robx",				R, 0, O_ROBX },
-    { "spam_cutoff",			R, 0, O_SPAM_CUTOFF },
-    { "spam_header_name",		R, 0, O_SPAM_HEADER_NAME },
-    { "spam_subject_tag",		R, 0, O_SPAM_SUBJECT_TAG },
-    { "spamicity_formats",		R, 0, O_SPAMICITY_FORMATS },
-    { "spamicity_tags",			R, 0, O_SPAMICITY_TAGS },
-    { "stats_in_header",		R, 0, O_STATS_IN_HEADER },
+    { "spam-cutoff",			R, 0, O_SPAM_CUTOFF },
+    { "spam-header-name",		R, 0, O_SPAM_HEADER_NAME },
+    { "spam-subject-tag",		R, 0, O_SPAM_SUBJECT_TAG },
+    { "spamicity-formats",		R, 0, O_SPAMICITY_FORMATS },
+    { "spamicity-tags",			R, 0, O_SPAMICITY_TAGS },
+    { "stats-in-header",		R, 0, O_STATS_IN_HEADER },
     { "terse",				R, 0, O_TERSE },
-    { "terse_format",			R, 0, O_TERSE_FORMAT },
-    { "thresh_update",			R, 0, O_THRESH_UPDATE },
+    { "terse-format",			R, 0, O_TERSE_FORMAT },
+    { "thresh-update",			R, 0, O_THRESH_UPDATE },
     { "timestamp",			R, 0, O_TIMESTAMP },
-    { "unsure_subject_tag",		R, 0, O_UNSURE_SUBJECT_TAG },
-    { "user_config_file",		R, 0, O_USER_CONFIG_FILE },
+    { "unsure-subject-tag",		R, 0, O_UNSURE_SUBJECT_TAG },
     { "wordlist",			R, 0, O_WORDLIST },
+    /* end of list */
     { NULL,				0, 0, 0 }
 };
 
+/*---------------------------------------------------------------------------*/
+
 static bool get_bool(const char *name, const char *arg)
 {
     bool b = str_to_bool(arg);
@@ -189,7 +188,7 @@ void process_parameters(int argc, char **argv, bool warn_on_error)
 #endif
 
     process_arglist(argc, argv, PR_COMMAND, PASS_1_CLI);
-    process_config_files(warn_on_error);
+    process_config_files(warn_on_error, longopts_bogofilter);
     process_arglist(argc, argv, PR_COMMAND, PASS_3_CLI);
 
     /* directories from command line and config file are already handled */
@@ -200,6 +199,11 @@ void process_parameters(int argc, char **argv, bool warn_on_error)
 	exit(EX_ERROR);
     }
 
+    if (!config_setup) {
+	/* set up default wordlist */
+	init_wordlist("word", WORDLIST, 0, WL_REGULAR);
+    }
+
     stats_prefix= stats_in_header ? "  " : "# ";
 
     return;
@@ -258,7 +262,7 @@ static int validate_args(void)
 		      outfname);
     }
     
-    if (run_register && (run_classify || (Rtable != 0)))
+    if (run_register && (run_classify || Rtable))
     {
 	(void)fprintf(stderr,
 		      "Error:  Option '-u' may not be used with options '-s', '-n', '-S', or '-N'.\n"
@@ -271,7 +275,7 @@ static int validate_args(void)
 
 static const char *help_text[] = {
     "help options:\n",
-    "  -h,                       - print this help message.\n",
+    "  -h, --help                - print this help message.\n",
     "  -V, --version             - print version information and exit.\n",
     "  -Q, --query               - query (display) base bogofilter configuration.\n",
     "  -QQ                       - display extended configuration info.\n",
@@ -279,8 +283,8 @@ static const char *help_text[] = {
     "  -p, --passthrough         - passthrough.\n",
     "  -e, --ham-true            - in -p mode, exit with code 0 when the mail is not spam.\n",
     "  -u, --update-as-scored    - score message as spam or non-spam and register accordingly.\n",
-    "  -M, --clasify-mbox        - set mailbox mode.  Classify multiple messages in an mbox formatted file.\n",
-    "  -b, --clasify-stdin       - set streaming bulk mode. Process multiple messages (files or directories) read from STDIN.\n",
+    "  -M, --classify-mbox       - set mailbox mode.  Classify multiple messages in an mbox formatted file.\n",
+    "  -b, --classify-stdin      - set streaming bulk mode. Process multiple messages (files or directories) read from STDIN.\n",
     "  -B, --classify-files=list - set bulk mode. Process multiple messages (files or directories) named on the command line.\n",
     "  -R, --dataframe           - print an R data frame.\n",
     "registration options:\n",
@@ -291,20 +295,18 @@ static const char *help_text[] = {
     "general options:\n",
     "  -c, --config-file=file    - read specified config file.\n",
     "  -C, --no-config-file      - don't read standard config files.\n",
-    "  -d, --bogofilter_dir=path - specify directory for wordlists.\n",
+    "  -d, --bogofilter-dir=path - specify directory for wordlists.\n",
     "  -H, --no-header-tags      - disables header line tagging.\n",
-    "  -k, --db_cachesize=size   - set BerkeleyDB cache size (MB).\n",
+    "  -k, --db-cachesize=size   - set Berkeley DB cache size (MB).\n",
     "  -l, --use-syslog          - write messages to syslog.\n",
     "  -L, --syslog-tag=tag      - specify the tag value for log messages.\n",
     "  -I, --input-file=file     - read message from 'file' instead of stdin.\n",
     "  -O, --output-file=file    - save message to 'file' in passthrough mode.\n",
     "parameter options:\n",
-    "  -mv1[,v2[,v3]]\n",
-    "      --min_dev=v1, --robs=v2, --robx=v3\n",
-    "                            - set user defined min_dev, robs, and robx values.\n",
-    "  -ov1[,v2]\n",
-    "      --spam_cutoff=v1, --ham_cutoff=v2\n",
-    "                            - set user defined spam and non-spam cutoff values.\n",
+    "  -mv1[,v2[,v3]]            - set user defined min_dev, robs, and robx values.\n",
+    "      --min-dev=v1, --robs=v2, --robx=v3\n",
+    "  -ov1[,v2]                 - set user defined spam and non-spam cutoff values.\n",
+    "      --spam-cutoff=v1, --ham-cutoff=v2\n",
     "info options:\n",
     "  -t, --terse               - set terse output mode.\n",
     "  -T, --fixed-terse-format  - set invariant terse output mode.\n",
@@ -316,32 +318,32 @@ static const char *help_text[] = {
     "config file options:\n",
     "  --option=value - can be used to set the value of a config file option.\n",
     "                   see bogofilter.cf.example for more info.\n",
-    "  --block_on_subnets                return class addr tokens\n",
-    "  --bogofilter_dir                  directory for wordlists\n",
-    "  --charset_default                 default character set\n",
-    "  --db_cachesize                    Berkeley db cache in Mb\n",
-    "  --ham_cutoff                      nonspam if score below this\n",
-    "  --header_format                   spam header format\n",
-    "  --log_header_format               header written to log\n",
-    "  --log_update_format               logged on update\n",
-    "  --min_dev                         ignore if score near\n",
-    "  --ns_esf                          effective size factor for ham\n",
-    "  --replace_nonascii_characters     substitute '?' if bit8 is 1\n",
+    "  --block-on-subnets                return class addr tokens\n",
+    "  --bogofilter-dir                  directory for wordlists\n",
+    "  --charset-default                 default character set\n",
+    "  --db-cachesize                    Berkeley db cache in Mb\n",
+    "  --ham-cutoff                      nonspam if score below this\n",
+    "  --header-format                   spam header format\n",
+    "  --log-header-format               header written to log\n",
+    "  --log-update-format               logged on update\n",
+    "  --min-dev                         ignore if score near\n",
+    "  --ns-esf                          effective size factor for ham\n",
+    "  --replace-nonascii-characters     substitute '?' if bit8 is 1\n",
     "  --robs                            Robinson's s parameter\n",
     "  --robx                            Robinson's x parameter\n",
-    "  --sp_esf                          effective size factor for spam\n",
-    "  --spam_cutoff                     spam if score above this\n",
-    "  --spam_header_name                passthrough adds/replaces\n",
-    "  --spam_subject_tag                passthrough prepends Subject\n",
-    "  --spamicity_formats               spamicity output format\n",
-    "  --spamicity_tags                  spamicity tag format\n",
-    "  --stats_in_header                 use header not body\n",
+    "  --sp-esf                          effective size factor for spam\n",
+    "  --spam-cutoff                     spam if score above this\n",
+    "  --spam-header-name                passthrough adds/replaces\n",
+    "  --spam-subject-tag                passthrough prepends Subject\n",
+    "  --spamicity-formats               spamicity output format\n",
+    "  --spamicity-tags                  spamicity tag format\n",
+    "  --stats-in-header                 use header not body\n",
     "  --terse                           report in short form\n",
-    "  --terse_format                    short form\n",
-    "  --thresh_update                   no update if near 0 or 1\n",
+    "  --terse-format                    short form\n",
+    "  --thresh-update                   no update if near 0 or 1\n",
     "  --timestamp                       apply token timestamps\n",
-    "  --unsure_subject_tag              like spam_subject_tag\n",
-    "  --user_config_file                configuration file\n",
+    "  --unsure-subject-tag              like spam-subject-tag\n",
+    "  --user-config-file                configuration file\n",
     "  --wordlist                        specify wordlist parameters\n",
     "\n",
     "bogofilter is a tool for classifying email as spam or non-spam.\n",
@@ -370,7 +372,7 @@ static void print_version(void)
 		  "%s version %s\n"
 		  "    Database: %s\n"
 		  "Copyright (C) 2002-2004 Eric S. Raymond,\n"
-		  "David Relson, Matthias Andree, Greg Louis\n\n"
+		  "    David Relson, Matthias Andree, Greg Louis\n\n"
 		  "%s comes with ABSOLUTELY NO WARRANTY.  "
 		  "This is free software, and\nyou are welcome to "
 		  "redistribute it under the General Public License.  "
@@ -380,7 +382,7 @@ static void print_version(void)
 		  progtype, version, ds_version_str(), PACKAGE);
 }
 
-#define	OPTIONS	":-:bBc:Cd:DeFhHI:k:lL:m:MnNo:O:pqQRsStTuUvVx:X:y:"
+#define	OPTIONS	":-:bBc:Cd:DehHI:k:lL:m:MnNo:O:pqQRsStTuUvVx:X:y:"
 
 /** These functions process command line arguments.
  **
@@ -393,7 +395,6 @@ static void print_version(void)
 
 static void process_arglist(int argc, char **argv, priority_t precedence, int pass)
 {
-    int option;
     ex_t exitcode;
 
     if (pass != PASS_1_CLI) {
@@ -409,17 +410,18 @@ static void process_arglist(int argc, char **argv, priority_t precedence, int pa
 
     while (1) 
     {
+	int option;
 	int option_index = 0;
 	int this_option_optind = optind ? optind : 1;
 	const char *name;
 
 	option = getopt_long(argc, argv, OPTIONS,
-			     long_options, &option_index);
+			     longopts_bogofilter, &option_index);
 
 	if (option == -1)
 	    break;
 
-	name = (option_index == 0) ? argv[this_option_optind] : long_options[option_index].name;
+	name = (option_index == 0) ? argv[this_option_optind] : longopts_bogofilter[option_index].name;
 
 #ifdef EXCESSIVE_DEBUG
 	if (getenv("BOGOFILTER_DEBUG_OPTIONS")) {
@@ -475,8 +477,9 @@ void process_arg(int option, const char *name, const char *val, priority_t prece
 	break;
 
     case 'c':
+    case O_CONFIG_FILE:
 	if (pass == PASS_1_CLI) {
-	    if (!read_config_file(val, false, !quiet, PR_CFG_USER)) {
+	    if (!read_config_file(val, false, !quiet, PR_CFG_USER, longopts_bogofilter)) {
 		fprintf(stderr, "Cannot open %s: %s\n", val, strerror(errno));
 		exit(EX_ERROR);
 	    }
@@ -591,12 +594,12 @@ void process_arg(int option, const char *name, const char *val, priority_t prece
     }
 	
     case ':':
-	fprintf(stderr, "Option -%c requires an argument.\n", optopt);
+	fprintf(stderr, "Option %s requires an argument.\n", name);
 	exit(EX_ERROR);
 
     case '-':
 	if (pass == PASS_3_CLI)
-	    process_config_option(val, true, precedence);
+	    process_config_option(val, true, precedence, longopts_bogofilter);
 	break;
 
     case 'd':
@@ -694,6 +697,12 @@ void process_arg(int option, const char *name, const char *val, priority_t prece
     case O_TIMESTAMP:			timestamp_tokens = get_bool(name, val);			break;
     case O_UNSURE_SUBJECT_TAG:		unsure_subject_tag = get_string(name, val);		break;
     case O_WORDLIST:			configure_wordlist(val);				break;
+
+    case O_DB_TRANSACTION:		fTransaction = get_bool(name, val);			break;
+
+    default:
+	if (!dsm_options_bogofilter(option, name, val))
+	    abort();
     }
 }
 
@@ -703,7 +712,7 @@ void process_arg(int option, const char *name, const char *val, priority_t prece
 #define YN(b) (b ? "Yes" : "No")
 #define NB(b) ((b != NULL && *b != '\0') ? b : "''")
 
-void query_config(void)
+rc_t query_config(void)
 {
     Q1 fprintf(stdout, "# %s version %s\n", progname, version);
     Q1 fprintf(stdout, "\n");
@@ -715,41 +724,50 @@ void query_config(void)
     Q1 fprintf(stdout, "%-11s = %0.6f  # (%8.2e)\n", "ns_esf", ns_esf, ns_esf);
     Q1 fprintf(stdout, "%-11s = %0.6f  # (%8.2e)\n", "sp_esf", sp_esf, sp_esf);
     Q1 fprintf(stdout, "\n");
-    Q1 fprintf(stdout, "%-17s = %s\n",    "block_on_subnets",    YN(block_on_subnets));
-    Q1 fprintf(stdout, "%-17s = %s\n",    "charset_default",     charset_default);
-    Q1 fprintf(stdout, "%-17s = %s\n",    "replace_nonascii_characters", YN(replace_nonascii_characters));
+    Q1 fprintf(stdout, "%-17s = %s\n",    "block-on-subnets",    YN(block_on_subnets));
+    Q1 fprintf(stdout, "%-17s = %s\n",    "charset-default",     charset_default);
+    Q1 fprintf(stdout, "%-17s = %s\n",    "replace-nonascii-characters", YN(replace_nonascii_characters));
     Q2 fprintf(stdout, "%-17s = %s\n",    "no-header-tags",      YN(header_line_markup));
-    Q1 fprintf(stdout, "%-17s = %s\n",    "stats_in_header",     YN(stats_in_header));
+    Q1 fprintf(stdout, "%-17s = %s\n",    "stats-in-header",     YN(stats_in_header));
     Q2 fprintf(stdout, "%-17s = %s\n",    "report-unsure",       YN(unsure_stats));
-    Q1 fprintf(stdout, "%-17s = %0.6f\n", "thresh_update",       thresh_update);
+    Q1 fprintf(stdout, "%-17s = %0.6f\n", "thresh-update",       thresh_update);
     Q1 fprintf(stdout, "%-17s = %s\n",    "timestamp",           YN(timestamp_tokens));
     Q2 fprintf(stdout, "%-17s = %d\n",    "timestamp-date",      today);
     Q1 fprintf(stdout, "\n");
-    Q1 fprintf(stdout, "%-17s = %s\n", "spam_header_name",    spam_header_name);
-    Q1 fprintf(stdout, "%-17s = %s\n", "spam_subject_tag",    NB(spam_subject_tag));
-    Q1 fprintf(stdout, "%-17s = %s\n", "unsure_subject_tag",  NB(unsure_subject_tag));
+    Q1 fprintf(stdout, "%-17s = %s\n", "spam-header-name",    spam_header_name);
+    Q1 fprintf(stdout, "%-17s = %s\n", "spam-subject-tag",    NB(spam_subject_tag));
+    Q1 fprintf(stdout, "%-17s = %s\n", "unsure-subject-tag",  NB(unsure_subject_tag));
     Q2 fprintf(stdout, "%-18s = %s\n", "syslog-tag",          NB(logtag));
-    Q1 fprintf(stdout, "%-17s = %s\n", "header_format",       header_format);
-    Q1 fprintf(stdout, "%-17s = %s\n", "terse_format",        terse_format);
-    Q1 fprintf(stdout, "%-17s = %s\n", "log_header_format",   log_header_format);
-    Q1 fprintf(stdout, "%-17s = %s\n", "log_update_format",   log_update_format);
-    Q1 display_tag_array("spamicity_tags   ", spamicity_tags);
-    Q1 display_tag_array("spamicity_formats", spamicity_formats);
+    Q1 fprintf(stdout, "%-17s = %s\n", "header-format",       header_format);
+    Q1 fprintf(stdout, "%-17s = %s\n", "terse-format",        terse_format);
+    Q1 fprintf(stdout, "%-17s = %s\n", "log-header-format",   log_header_format);
+    Q1 fprintf(stdout, "%-17s = %s\n", "log-update-format",   log_update_format);
+    Q1 display_tag_array("spamicity-tags   ", spamicity_tags);
+    Q1 display_tag_array("spamicity-formats", spamicity_formats);
 
     Q2 fprintf(stdout, "\n");
 
     Q2 fprintf(stdout, "%-18s = %s\n", "no-config-file",   YN(suppress_config_file));
     Q2 fprintf(stdout, "%-18s = %s\n", "config-file",      NB(config_file_name));
-    Q2 fprintf(stdout, "%-18s = %s\n", "user_config_file", NB(user_config_file));
+    Q2 fprintf(stdout, "%-18s = %s\n", "user-config-file", NB(user_config_file));
     Q2 fprintf(stdout, "\n");
 
-    Q2 fprintf(stdout, "%-18s = %s\n", "bogofilter_dir", bogohome);
-    Q2 display_wordlists("%-18s   ");
+    Q2 fprintf(stdout, "%-18s = %s\n", "bogofilter-dir", 	bogohome);
+    Q2 display_wordlists(word_lists, "%-18s   ");
     Q2 fprintf(stdout, "\n");
 
-    Q2 fprintf(stdout, "%-18s = %d\n", "db_cachesize",    db_cachesize);
+    Q2 fprintf(stdout, "%-18s = %d\n", "db-cachesize",        	db_cachesize);
 
-    exit(EX_OK);
+#ifdef	HAVE_DECL_DB_CREATE
+    Q2 fprintf(stdout, "%-18s = %d\n", "db-lk-max-locks",   	db_max_locks);
+    Q2 fprintf(stdout, "%-18s = %d\n", "db-lk-max-objects", 	db_max_objects);
+    Q2 fprintf(stdout, "%-18s = %s\n", "db-log-autoremove",     YN(db_log_autoremove));
+#ifdef	FUTURE_DB_OPTIONS
+    Q2 fprintf(stdout, "%-18s = %s\n", "db-log-txn-durable",	YN(db_txn_durable));
+#endif
+#endif
+
+    return RC_OK;
 }
 
 static void display_tag_array(const char *label, FIELD *array)
