@@ -1,4 +1,4 @@
-/* $Id: bogoconfig.h,v 1.20 2004/08/22 15:43:14 relson Exp $ */
+/* $Id: bogoconfig.h,v 1.22 2005/01/06 14:46:24 m-a Exp $ */
 
 /*****************************************************************************
 
@@ -22,10 +22,9 @@ extern const char *logtag;
 extern const char *spam_header_name;
 extern const char *user_config_file;
 
-extern void query_config(void) __attribute__ ((noreturn));
+extern rc_t query_config(void);
 extern void process_parameters(int argc, char **argv, bool warn_on_error);
 
-extern struct option long_options[];
 extern void process_arg(int option, const char *name, const char *arg, priority_t precedence, arg_pass_t pass);
 
 #endif
