@@ -1,4 +1,4 @@
-/* $Id: bogolexer.c,v 1.75 2004/08/30 19:35:34 relson Exp $ */
+/* $Id: bogolexer.c,v 1.85 2005/03/23 23:57:13 m-a Exp $ */
 
 /*****************************************************************************
 
@@ -23,6 +23,7 @@ AUTHOR:
 #include "charset.h"
 #include "configfile.h"
 #include "lexer.h"
+#include "longoptions.h"
 #include "mime.h"
 #include "textblock.h"
 #include "token.h"
@@ -56,6 +57,7 @@ static void help(void)
 	    "\t-C\t- don't read standard config files.\n"
 	    "\t-H\t- disables header line tagging.\n"
 	    "\t-I file\t- read message from file instead of stdin.\n"
+	    "\t-O file\t- write to file instead of stdout.\n"
 	    "\t-x list\t- set debug flags.\n"
 	    "\t-D\t- direct debug output to stdout.\n");
     fprintf(stderr,
@@ -78,42 +80,6 @@ static void print_version(void)
 		  progname, version, PACKAGE);
 }
 
-struct option long_options[] = {
-    { "config-file",			N, 0, 'c' },
-    { "no-config-file",			N, 0, 'C' },
-    { "debug-flags",			R, 0, 'x' },
-    { "debug-to-stdout",		N, 0, 'D' },
-    { "no-header-tags",			N, 0, 'H' },
-    { "input-file",			N, 0, 'I' },
-    { "output-file",			N, 0, 'O' },
-    { "query",				N, 0, 'Q' },
-    { "verbosity",			N, 0, 'v' },
-    { "block_on_subnets",		R, 0, O_BLOCK_ON_SUBNETS },
-    { "bogofilter_dir",			R, 0, O_IGNORE },
-    { "charset_default",		R, 0, O_IGNORE },
-    { "ham_cutoff",			R, 0, O_IGNORE },
-    { "header_format",			R, 0, O_IGNORE },
-    { "log_header_format",		R, 0, O_IGNORE },
-    { "log_update_format",		R, 0, O_IGNORE },
-    { "min_dev",			R, 0, O_IGNORE },
-    { "replace_nonascii_characters",	R, 0, 'n' },
-    { "robs",				R, 0, O_IGNORE },
-    { "robx",				R, 0, O_IGNORE },
-    { "spam_cutoff",			R, 0, O_IGNORE },
-    { "spam_header_name",		R, 0, O_IGNORE },
-    { "spam_subject_tag",		R, 0, O_IGNORE },
-    { "spamicity_formats",		R, 0, O_IGNORE },
-    { "spamicity_tags",			R, 0, O_IGNORE },
-    { "stats_in_header",		R, 0, O_IGNORE },
-    { "terse",				R, 0, O_IGNORE },
-    { "terse_format",			R, 0, O_IGNORE },
-    { "thresh_update",			R, 0, O_IGNORE },
-    { "timestamp",			R, 0, O_IGNORE },
-    { "unsure_subject_tag",		R, 0, O_IGNORE },
-    { "user_config_file",		R, 0, O_USER_CONFIG_FILE },
-    { NULL,				0, 0, 0 }
-};
-
 static bool get_bool(const char *name, const char *arg)
 {
     bool b = str_to_bool(arg);
@@ -131,7 +97,16 @@ static char *get_string(const char *name, const char *arg)
     return s;
 }
 
-#define	OPTIONS	":c:CDhHI:npqvVx:X:m"
+static struct option longopts_bogolexer[] = {
+    /* longoptions.h - common options */
+    LONGOPTIONS_COMMON
+    /* longoptions.h - options for bogofilter and bogolexer */
+    LONGOPTIONS_LEX
+    /* end of list */
+    { NULL,				0, 0, 0 }
+};
+
+#define	OPTIONS	":c:CDhHI:nO:pqvVx:X:m"
 
 /** These functions process command line arguments.
  **
@@ -165,12 +140,12 @@ static void process_arglist(int argc, char **argv)
 #endif
 
 	option = getopt_long(argc, argv, OPTIONS,
-			     long_options, &option_index);
+			     longopts_bogolexer, &option_index);
 
 	if (option == -1)
 	    break;
 
-	name = (option_index == 0) ? argv[this_option_optind] : long_options[option_index].name;
+	name = (option_index == 0) ? argv[this_option_optind] : longopts_bogolexer[option_index].name;
 	process_arg(option, name, optarg, PR_COMMAND, PASS_1_CLI);
     }
 
@@ -188,7 +163,7 @@ void process_arg(int option, const char *name, const char *val, priority_t prece
     switch (option)
     {
     case ':':
-	fprintf(stderr, "Option -%c requires an argument.\n", optopt);
+	fprintf(stderr, "Option %s requires an argument.\n", name);
 	exit(EX_ERROR);
 
     case '?':
@@ -196,7 +171,8 @@ void process_arg(int option, const char *name, const char *val, priority_t prece
 	break;
 
     case 'c':
-	read_config_file(val, false, false, precedence);
+    case O_CONFIG_FILE:
+	read_config_file(val, false, false, precedence, longopts_bogolexer);
 	/*@fallthrough@*/
 	/* fall through to suppress reading config files */
 
@@ -224,12 +200,20 @@ void process_arg(int option, const char *name, const char *val, priority_t prece
 	bogoreader_name(val);
 	break;
 
+    case 'O':
+	fpo = fopen(val, "wt");
+	break;
+
     case 'n':
 	replace_nonascii_characters = true;
 	break;
 
+    case O_CHARSET_DEFAULT:
+	charset_default = get_string(name, val);
+	break;
+
     case O_REPLACE_NONASCII_CHARACTERS:
-	replace_nonascii_characters = get_bool(name, val);	
+	replace_nonascii_characters = get_bool(name, val);
 	break;
 
     case 'p':
@@ -278,39 +262,42 @@ int main(int argc, char **argv)
 {
     token_t t;
 
+    fpo = stdout;
+
     mbox_mode = true;		/* to allow multiple messages */
 
     process_arglist(argc, argv);
-    process_config_files(false);
+    process_config_files(false, longopts_bogolexer);
 
     textblock_init();
 
     if (!passthrough)
     {
 	if (quiet)
-	    puts( "quiet mode.");
+	    fprintf(fpo, "quiet mode.\n");
 	else
-	    puts("normal mode.");
+	    fprintf(fpo, "normal mode.\n");
     }
 
     bogoreader_init(argc, argv);
 
     while ((*reader_more)()) {
+	word_t *token;
 	lexer_init();
 
-	while ((t = get_token()) != NONE)
+	while ((t = get_token( &token )) != NONE)
 	{
 	    count += 1;
 	    if (passthrough) {
-		fprintf(stdout, "%s\n", yylval->text);
+		fprintf(fpo, "%s\n", token->text);
 	    }
 	    else if (!quiet)
-		printf("get_token: %d \"%s\"\n", (int)t, yylval->text);
+		fprintf(fpo, "get_token: %d \"%s\"\n", (int)t, token->text);
 	}
     }
 
     if ( !passthrough )
-	printf( "%d tokens read.\n", count );
+	fprintf(fpo, "%d tokens read.\n", count);
 
     textblock_free();
 
@@ -318,4 +305,3 @@ int main(int argc, char **argv)
 
     return 0;
 }
-
