@@ -1,4 +1,4 @@
-/* $Id: bogotune.c,v 1.167 2004/10/19 23:49:47 relson Exp $ */
+/* $Id: bogotune.c,v 1.210 2005/04/07 00:01:51 relson Exp $ */
 
 /*****************************************************************************
 
@@ -47,19 +47,19 @@ AUTHORS:
 #include <errno.h>
 #include <stdlib.h>
 #include <sys/stat.h>
+#include <unistd.h>
 
 #include "bogotune.h"
 
 #include "bogoconfig.h"
-#include "bogohome.h"
 #include "bogoreader.h"
 #include "collect.h"
 #include "datastore.h"
 #include "msgcounts.h"
 #include "mime.h"
+#include "mxcat.h"
 #include "paths.h"
 #include "robx.h"
-#include "rstats.h"
 #include "score.h"
 #include "token.h"
 #include "tunelist.h"
@@ -71,10 +71,10 @@ AUTHORS:
 #undef	HAM_CUTOFF	/* ignore value in score.h */
 #undef	SPAM_CUTOFF	/* ignore value in score.h */
 
-#define	TEST_COUNT	500	/* minimum allowable message count */
-#define	LIST_COUNT	2000	/* minimum msg count in tunelist   */
-#define	PREF_COUNT	4000	/* preferred message count         */
-#define	LARGE_COUNT	40000
+#define	TEST_COUNT	500u	/* minimum allowable message count */
+#define	LIST_COUNT	2000u	/* minimum msg count in tunelist   */
+#define	PREF_COUNT	4000u	/* preferred message count         */
+#define	LARGE_COUNT	40000u
 
 #define	HAM_CUTOFF	0.10
 #define MIN_HAM_CUTOFF	0.10	/* minimum final ham_cutoff */
@@ -91,8 +91,8 @@ AUTHORS:
 				** and low scoring spam */
 
 /* bogotune's default parameters */
-#define	DEFAULT_ROBS	ROBS	/* 0.010 */
-#define	DEFAULT_ROBX	ROBX	/* 0.415 */
+#define	DEFAULT_ROBS	ROBS	/* 0.0178 */		
+#define	DEFAULT_ROBX	ROBX	/* 0.5200 */
 #define	DEFAULT_MIN_DEV	0.02
 
 /* coarse scan parms */
@@ -136,15 +136,18 @@ typedef enum e_ds_loc {
 /* Global Variables */
 
 const char *progname = "bogotune";
-static char *ds_file;
-static char *ds_path;
+static char *ds_path;			/* directory */
+static char *ds_file;			/* directory/file */
 static ds_loc ds_flag = DS_NONE;
+static void *env = NULL;
 
-static bool    bogolex = false;		/* true if convert input to msg-count format */
 static bool    esf_flag = true;		/* test ESF factors if true */
-static char   *bogolex_file = NULL;
+static bool    exit_zero = false;	/* non-error exits zero */
+static char   *bogolex_file = NULL;	/* non-NULL if creating msg-count output */
 static word_t *w_msg_count;
 
+static uint message_count;
+
 static tunelist_t *ns_and_sp;
 static tunelist_t *ns_msglists, *sp_msglists;
 
@@ -179,15 +182,8 @@ static uint   ncnt, nsum;		/* neighbor count and sum - for gfn() averaging */
 uint test = 0;
 #endif
 
-/* Function Prototypes */
-
-static void bt_exit(void);
-static void check_wordlist_path(void);
-static int  load_hook(word_t *key, dsv_t *data, void *userdata);
-static void load_wordlist(ds_foreach_t *hook, void *userdata);
-static void show_elapsed_time(int beg, int end, uint cnt, double val,
-			      const char *lbl1, const char *lbl2);
-static void write_msgcount_file(wordhash_t *wh);
+bool fMakeCheck = false;	/* allows quick & dirty regression testing */
+uint cMakeCheck =    50;	/* ... for 50 cycles */
 
 /* Function Definitions */
 
@@ -374,7 +370,8 @@ static void score_ns(double *results)
 	    wordhash_t *wh = item->wh;
 	    double score = msg_compute_spamicity(wh, NULL);
 	    results[count++] = score;
-	    if (-verbose >= SCORE_DETAIL)
+	    if ( -verbose == SCORE_DETAIL ||
+		(-verbose >= SCORE_DETAIL && EPS < score && score < 1 - EPS))
 		printf("%6u %0.16f\n", count-1, score);
 	}
     }
@@ -423,7 +420,8 @@ static void score_sp(double *results)
 	    wordhash_t *wh = item->wh;
 	    double score = msg_compute_spamicity(wh, NULL);
 	    results[count++] = score;
-	    if (-verbose >= SCORE_DETAIL)
+	    if ( -verbose == SCORE_DETAIL ||
+		(-verbose >= SCORE_DETAIL && EPS < score && score < 1 - EPS))
 		printf("%6u %0.16f\n", count-1, score);
 	}
     }
@@ -601,9 +599,121 @@ static void set_thresh(uint count, double *scores)
     return;
 }
 
+static void init_count(void)
+{
+    message_count = 0;
+}
+
+static void print_final_count(void)
+{
+    if (verbose) {
+	if (!fMakeCheck)
+	    printf("\r              \r");
+	printf("%u messages\n", message_count);
+	fflush(stdout);
+    }
+}
+
+static int update_count(void)
+{
+    message_count += 1;
+
+    if (verbose && (message_count % 100) == 0 && !fMakeCheck) {
+	if ((message_count % 1000) != 0)
+	    putchar('.');
+	else
+	    printf("\r              \r%u ", message_count/1000 );
+	fflush(stdout);
+    }
+    return message_count;
+}
+
+static void load_wordlist(ds_foreach_t *hook, void *userdata)
+{
+    bfpath *bfp = bfpath_create(ds_path);
+
+    if (!bfpath_check_mode(bfp, BFP_MUST_EXIST)) {
+	fprintf(stderr, "Can't open wordlist '%s'\n", bfp->filepath);
+	exit(EX_ERROR);
+    }
+
+    if (verbose) {
+	printf("Reading %s\n", ds_path);
+	fflush(stdout);
+    }
+
+    ds_oper(env, bfp, DS_READ, hook, userdata);
+
+    bfpath_free(bfp);
+
+    return;
+}
+
+static int load_hook(word_t *key, dsv_t *data, void *userdata)
+/* returns 0 if ok, 1 if not ok */
+{
+    wordhash_t *train = userdata;
+
+    wordprop_t *tokenprop = wordhash_insert(train, key, sizeof(wordprop_t), &wordprop_init);
+    tokenprop->cnts.bad = data->spamcount;
+    tokenprop->cnts.good = data->goodcount;
+
+    if (word_cmps(key, ".MSG_COUNT") == 0)
+	set_msg_counts(data->goodcount, data->spamcount);
+
+    return 0;
+}
+
+static void set_train_msg_counts(wordhash_t *train, wordhash_t *wh)
+{
+    wordprop_t *count;
+    count = wordhash_insert(wh, w_msg_count, sizeof(wordprop_t), NULL);
+    if (count->cnts.good == 0 || count->cnts.bad == 0)
+	load_wordlist(load_hook, train);
+    if (msgs_good == 0 && msgs_bad == 0) {
+	fprintf(stderr, "Can't find '.MSG_COUNT'.\n");
+	exit(EX_ERROR);
+    }
+}
+
+/* write_msgcount_file()
+**
+**	Create a message count file from the original messages
+*/
+
+static void print_msgcount_entry(const char *token, uint bad, uint good)
+{
+    printf( "\"%s\" %u %u\n", token, bad, good);
+}
+
+static void write_msgcount_file(wordhash_t *wh)
+{
+    hashnode_t *hn;
+    wordhash_t *train = ns_and_sp->train;
+
+    print_msgcount_entry(".MSG_COUNT", msgs_bad, msgs_good);
+
+    for (hn = wordhash_first(wh); hn != NULL; hn = wordhash_next(wh)) {
+	word_t *token = hn->key;
+	wordprop_t *wp = (wordprop_t *) hn->buf;
+	wordcnts_t *cnts = &wp->cnts;
+
+	if (cnts->good == 0 && cnts->bad == 0) {
+	    wp = wordhash_search(train, token, 0);
+	    if (wp) {
+		cnts->good = wp->cnts.good;
+		cnts->bad  = wp->cnts.bad;
+	    }
+	}
+
+	print_msgcount_entry((char *)token->text, cnts->bad, cnts->good);
+    }
+
+    return;
+}
+
 static uint read_mailbox(char *arg, mlhead_t *msgs)
 {
-    uint count = 0;
     wordhash_t *train = ns_and_sp->train;
 
     if (verbose) {
@@ -611,6 +721,7 @@ static uint read_mailbox(char *arg, mlhead_t *msgs)
 	fflush(stdout);
     }
 
+    init_count();
     mbox_mode = true;
     bogoreader_init(1, &arg);
     while ((*reader_more)()) {
@@ -619,57 +730,39 @@ static uint read_mailbox(char *arg, mlhead_t *msgs)
 
 	collect_words(whc);
 
-	if (ds_path != NULL && (int)(msgs_good + msgs_bad) == 0) {
-	    wordprop_t *msg_count;
-	    msg_count = wordhash_insert(whc, w_msg_count, sizeof(wordprop_t), NULL);
-	    if (msg_count->cnts.good == 0 || msg_count->cnts.bad == 0)
-		load_wordlist(load_hook, train);
-	    if (msgs_good == 0 && msgs_bad == 0) {
-		fprintf(stderr, "Can't find '.MSG_COUNT'.\n");
-		exit(EX_ERROR);
-	    }
-	}
+	if (ds_path != NULL && (msgs_good + msgs_bad) == 0)
+	    set_train_msg_counts(train, whc);
 
 	if (whc->count == 0 && !quiet) {
-	    printf("msg #%u, count is %u\n", count, whc->count);
+	    printf("msg #%u, count is %u\n", message_count, whc->count);
 	    bt_trap();
 	}
 
-	if (whc->count != 0) {
-	    count += 1;
-	    if (bogolex) {
-		write_msgcount_file(whc);
-	    }
-	    else {
-		if (!msg_count_file)
-		    whp = convert_wordhash_to_propslist(whc, train);
-		else
-		    whp = convert_propslist_to_countlist(whc);
-		msglist_add(msgs, whp);
-	    }
+	if (bogolex_file != NULL) {
+	    wordhash_sort(whc);
+	    lookup_words(whc);
+	    write_msgcount_file(whc);
+	}
+	else if (whc->count != 0) {
+	    if (!msg_count_file)
+		whp = convert_wordhash_to_propslist(whc, train);
+	    else
+		whp = convert_propslist_to_countlist(whc);
+	    msglist_add(msgs, whp);
 	}
 
+	update_count();
+	
 	if (whc != whp)
 	    wordhash_free(whc);
-
-	if (verbose && (count % 100) == 0) {
-	    if ((count % 1000) != 0)
-		putchar('.');
-	    else
-		printf("\r              \r%u ", count/1000 );
-	    fflush(stdout);
-	}
     }
 
-    ns_and_sp->count += count;
+    print_final_count();
+
+    ns_and_sp->count += message_count;
     bogoreader_fini();
 
-    if (verbose) {
-	printf("\r              \r%u messages\n", count);
-	fflush(stdout);
-    }
-
-    return count;
+    return message_count;
 }
 
 static uint filelist_read(int mode, flhead_t *list)
@@ -702,16 +795,17 @@ static void distribute(int mode, tunelist_t *ns_or_sp)
     int good = mode == REG_GOOD;
     int bad  = 1 - good;
 
-    bool divvy = ds_file == NULL && user_robx < EPS && !msg_count_file;
+    bool divvy = ds_flag == DS_RAM && user_robx < EPS && !msg_count_file;
 
-    wordhash_t *train = ns_and_sp->train;
-    mlhead_t *msgs = ns_or_sp->msgs;
     mlitem_t *item;
+    mlhead_t *msgs = ns_or_sp->msgs;
+    wordhash_t *train = ns_and_sp->train;
 
     int score_count = 0;
     int train_count = 0;
-    int train_good = 0;
-    int train_bad  = 0;
+
+    static int train_good = 0;
+    static int train_bad  = 0;
 
     double ratio = scale(msgs->count,
 			 LIST_COUNT + TEST_COUNT,	/* small count */
@@ -741,8 +835,7 @@ static void distribute(int mode, tunelist_t *ns_or_sp)
     }
 
     if (divvy) {
-	wordprop_t *msg_count;
-	msg_count = wordhash_insert(train, w_msg_count, sizeof(wordprop_t), &wordprop_init);
+	wordhash_insert(train, w_msg_count, sizeof(wordprop_t), &wordprop_init);
 	set_msg_counts(train_good, train_bad);
     }
 
@@ -758,7 +851,7 @@ static void create_countlists(tunelist_t *ns_or_sp)
 {
     uint i;
     uint c = COUNTOF(ns_or_sp->u.sets);
-    for (i = 0; i<c; i += 1) {
+    for (i = 0; i < c; i += 1) {
 	mlhead_t *list = ns_or_sp->u.sets[i];
 	mlitem_t *item;
 
@@ -775,44 +868,6 @@ static void create_countlists(tunelist_t *ns_or_sp)
     return;
 }
 
-/* write_msgcount_file()
-**
-**	Create a message count file from the original messages
-*/
-
-static void print_msgcount_entry(const char *token, uint bad, uint good)
-{
-    printf( "\"%s\" %u %u\n", token, bad, good);
-}
-
-static void write_msgcount_file(wordhash_t *wh)
-{
-    hashnode_t *hn;
-    wordhash_t *train = ns_and_sp->train;
-
-    print_msgcount_entry(".MSG_COUNT", (int) msgs_bad, (int) msgs_good);
-
-    wordhash_sort(wh);
-
-    for (hn = wordhash_first(wh); hn != NULL; hn = wordhash_next(wh)) {
-	word_t *token = hn->key;
-	wordprop_t *wp = (wordprop_t *) hn->buf;
-	wordcnts_t *cnts = &wp->cnts;
-
-	if (cnts->good == 0 && cnts->bad == 0) {
-	    wp = wordhash_search(train, token, 0);
-	    if (wp) {
-		cnts->good = wp->cnts.good;
-		cnts->bad  = wp->cnts.bad;
-	    }
-	}
-
-	print_msgcount_entry((char *)token->text, cnts->bad, cnts->good);
-    }
-
-    return;
-}
-
 static void print_version(void)
 {
     (void)fprintf(stderr,
@@ -831,7 +886,7 @@ static void print_version(void)
 static void help(void)
 {
     (void)fprintf(stderr,
-		  "Usage:  %s [options] { -c config } { -d directory } -n non-spam-files -s spam-files\n",
+		  "Usage:  %s [options] { -c config } { -d directory } -n non-spam-file -s spam-file\n",
 		  progname);
     (void)fprintf(stderr,
 		  "\t  -h      - print this help message.\n"
@@ -840,14 +895,13 @@ static void help(void)
 		  "\t  -D      - don't read a wordlist file.\n"
 		  "\t  -d path - specify directory for wordlists.\n"
 		  "\t  -E      - disable ESF (effective size factor) tuning.\n"
-		  "\t  -M      - output input file in message count format.\n"
+		  "\t  -M file - rewrite input file in message count format.\n"
 		  "\t  -r num  - specify robx value\n");
     (void)fprintf(stderr,
 		  "\t  -T num  - specify fp target value\n"
 		  "\t  -s file1 file2 ... - spam files\n"
 		  "\t  -n file1 file2 ... - non-spam files\n"
 		  "\t  -v      - increase level of verbose messages\n"
-		  "\t          - accept high scoring non-spam and low scoring spam\n"
 		  "\t  -q      - quiet (suppress warnings)\n"
 	);
     (void)fprintf(stderr,
@@ -856,6 +910,11 @@ static void help(void)
 		  progname, version);
 }
 
+static struct option longopts_bogotune[] = {
+    /* end of list */
+    { NULL,				0, 0, 0 }
+};
+
 static int process_arglist(int argc, char **argv)
 {
     int  count = 1;
@@ -867,80 +926,89 @@ static int process_arglist(int argc, char **argv)
     _wildcard (&argc, &argv);	/* expand wildcards (*.*) */
 #endif
 
-    while (--argc > 0) {
-	char *arg = *++argv;
-	count += 1;
-	if (*arg == '-') {
-	    while (*++arg) {
-		switch (*arg) {
-		case 'c':
-		    argc -= 1;
-		    read_config_file(*++argv, false, !quiet, PR_CFG_USER);
-		case 'C':
-		    suppress_config_file = true;
-		    break;
-		case 'd':
-		    argc -= 1;
-		    ds_file = *++argv;
-		    ds_flag = (ds_flag == DS_NONE) ? DS_DSK : DS_ERR;
-		    break;
-		case 'D':
-		    ds_flag = (ds_flag == DS_NONE) ? DS_RAM : DS_ERR;
-		    break;
-		case 'E':
-		    esf_flag ^= true;
-		    break;
-		case 'I':
-		    argc -= 1;
-		    bogolex_file = *++argv;
-		    break;
-		case 'M':
-		    bogolex = true;
-		    break;
-		case 'n':
-		    run_type = REG_GOOD;
-		    break;
-		case 'q':
-		    quiet = true;
-		    break;
-		case 'r':
-		    argc -= 1;
-		    user_robx = atof(*++argv);
-		    break;
-		case 's':
-		    run_type = REG_SPAM;
-		    break;
-		case 'v':
-		    verbose += 1;
-		    break;
+#define	OPTIONS	":c:Cd:DeEM:n:qr:s:tT:vVx:"
+
+    while (1)
+    {
+	int option;
+	
+	option = getopt(argc, argv, OPTIONS);
+
+	if (option == -1)
+ 	    break;
+
+	switch (option) {
+	case 'c':
+	    read_config_file(optarg, false, false, PR_CFG_USER, longopts_bogotune);
+	    /* FALLTHROUGH */
+	case 'C':
+	    suppress_config_file = true;
+	    break;
+	case 'd':
+	    ds_path = xstrdup(optarg);
+	    ds_flag = (ds_flag == DS_NONE) ? DS_DSK : DS_ERR;
+	    break;
+	case 'D':
+	    ds_flag = (ds_flag == DS_NONE) ? DS_RAM : DS_ERR;
+	    break;
+	case 'e':
+	    exit_zero = true;
+	    break;
+	case 'E':
+	    esf_flag ^= true;
+	    break;
+	case 'M':
+	    bogolex_file = optarg;
+	    break;
+	case 'n':
+	    run_type = REG_GOOD;
+	    filelist_add( ham_files, optarg );
+	    break;
+	case 'q':
+	    quiet = true;
+	    break;
+	case 'r':
+	    user_robx = atof(optarg);
+	    break;
+	case 's':
+	    run_type = REG_SPAM;
+	    filelist_add( spam_files, optarg );
+	    break;
 #ifdef	TEST
-		case 't':
-		    test += 1;
-		    break;
+	case 't':
+	    test += 1;
+	    break;
 #endif
-		case 'T':
-		    argc -= 1;
-		    coerced_target = atoi(*++argv);
-		    break;
-		case 'V':
-		    print_version();
-		    exit(EX_OK);
-		default:
-		    help();
-		    exit(EX_ERROR);
-		}
-	    }
+	case 'T':
+	    coerced_target = atoi(optarg);
+	    break;
+	case 'v':
+	    verbose += 1;
+	    break;
+	case 'V':
+	    print_version();
+	    exit(EX_OK);
+	case 'x':
+	    if (strcmp(optarg, "MakeCheck") == 0)
+		fMakeCheck = true;
+	    else
+		set_debug_mask( optarg );
+	    break;
+	default:
+	    help();
+	    exit(EX_ERROR);
 	}
-	else
-	    filelist_add( (run_type == REG_GOOD) ? ham_files : spam_files, arg);
     }
 
+    if (ds_flag == DS_NONE)	/* default is "wordlist on disk" */
+	ds_flag = DS_DSK;
+
     if (ds_flag == DS_ERR) {
 	fprintf(stderr, "Only one '-d dir' or '-D' option is allowed.\n");
 	exit(EX_ERROR);
     }
 
-    if (!bogolex &&
+    if (bogolex_file == NULL &&
 	(spam_files->count == 0 || ham_files->count == 0)) {
 	fprintf(stderr,
 		"Bogotune needs both non-spam and spam message sets for its parameter testing.\n");
@@ -948,72 +1016,25 @@ static int process_arglist(int argc, char **argv)
     }
 
     if (!suppress_config_file)
-	process_config_files(false);
+	process_config_files(false, longopts_bogotune);
 
     return count;
 }
 
-static void check_wordlist_path(void)
-{
-    struct stat sb;
-    size_t len = strlen(ds_file) + strlen(WORDLIST) + 2;
-    ds_path = xmalloc(len);
-
-    build_wordlist_path(ds_path, len, ds_file);
-
-    if (stat(ds_path, &sb) != 0) {
-	fprintf(stderr, "Error accessing file or directory '%s'.\n", ds_path);
-	if (errno != 0)
-	    fprintf(stderr, "error #%d - %s.\n", errno, strerror(errno));
-	return;
-    }
-
-    db_cachesize = ceil(sb.st_size / (3 * 1024 * 1024));
-
-    return;
-}
-
-static void load_wordlist(ds_foreach_t *hook, void *userdata)
-{
-    if (verbose) {
-	printf("Reading %s\n", ds_path);
-	fflush(stdout);
-    }
-
-    ds_init();
-    ds_oper(ds_path, DS_READ, hook, userdata);
-    ds_cleanup();
-
-    return;
-}
-
-static int load_hook(word_t *key, dsv_t *data, void *userdata)
-/* returns 0 if ok, 1 if not ok */
-{
-    wordhash_t *train = userdata;
-
-    wordprop_t *tokenprop = wordhash_insert(train, key, sizeof(wordprop_t), &wordprop_init);
-    tokenprop->cnts.bad = data->spamcount;
-    tokenprop->cnts.good = data->goodcount;
-
-    if (strcmp((char *)key->text, ".MSG_COUNT") == 0)
-	set_msg_counts(data->goodcount, data->spamcount);
-
-    return 0;
-}
-
 static double get_robx(void)
 {
     double rx;
 
     if (user_robx > 0.0)
 	rx = user_robx;
-    else {
+    else if (ds_flag == DS_DSK)	{
 	printf("Calculating initial x value...\n");
 	verbose = -verbose;		/* disable bogofilter debug output */
-	rx = compute_robinson_x(ds_file);
+	rx = compute_robinson_x();
 	verbose = -verbose;		/* enable bogofilter debug output */
     }
+    else
+	rx = ROBX;
 
     if (rx > RX_MAX) rx = RX_MAX;
     if (rx < RX_MIN) rx = RX_MIN;
@@ -1102,6 +1123,7 @@ static result_t *count_outliers(uint r_count, result_t *sorted, result_t *unsort
 	r = &sorted[i];
 	if (r->fp != target) continue;
 	if (j == 0) j = i+1;
+	if (fMakeCheck && j >= cMakeCheck) break;
 	rsi = r->rsi; mdi = r->mdi; rxi = r->rxi; spi = r->spi; nsi = r->nsi;
 	ncnt = nsum = 0;
 	if (((rsi == 0   ||
@@ -1276,7 +1298,7 @@ static void final_recommendations(bool skip)
 static void bogotune_init(void)
 {
     const char *msg_count = MSG_COUNT;
-    w_msg_count = word_new((const byte *)msg_count, strlen(msg_count));
+    w_msg_count = word_news(msg_count);
     ns_and_sp   = tunelist_new("tr");		/* training lists */
     ns_msglists = tunelist_new("ns");		/* non-spam scoring lists */
     sp_msglists = tunelist_new("sp");		/* spam     scoring lists */
@@ -1302,6 +1324,7 @@ static void bogotune_free(void)
     mime_cleanup();
 
     xfree(ds_path);
+    xfree(ds_file);
 
     return;
 }
@@ -1310,8 +1333,8 @@ static bool check_msgcount_parms(void)
 {
     bool ok = true;
 
-    if (ds_file == NULL) {
-	fprintf(stderr, "A wordlist directory must be specified for converting message to the message count format.\n");
+    if (ds_flag == DS_RAM) {
+	fprintf(stderr, "A wordlist directory must be specified for converting messages to the message count format.\n");
 	ok = false;
     }
 
@@ -1331,19 +1354,19 @@ static bool check_msg_counts(void)
     if (msgs_good < LIST_COUNT || msgs_bad < LIST_COUNT) {
 	if (!quiet)
 	    fprintf(stderr,
-		    "The wordlist contains %d non-spam and %d spam messages.\n"
-		    "Bogotune must be run with at least %d of each.\n",
-		    (int) msgs_good, (int) msgs_bad, LIST_COUNT);
+		    "The wordlist contains %u non-spam and %u spam messages.\n"
+		    "Bogotune must be run with at least %u of each.\n",
+		    msgs_good, msgs_bad, LIST_COUNT);
 	ok = false;
     }
 
-    if (msgs_bad < msgs_good / 5 ||
-	msgs_bad > msgs_good * 5) {
+    if (msgs_bad * 5.0 < msgs_good ||
+	msgs_bad > msgs_good * 5.0) {
 	if (!quiet)
 	    fprintf(stderr,
 		    "The wordlist has a ratio of spam to non-spam of %0.1f to 1.0.\n"
 		    "Bogotune requires the ratio be in the range of 0.2 to 5.\n",
-		    msgs_bad / msgs_good);
+		    (double)msgs_bad / msgs_good);
 	ok = false;
     }
 
@@ -1351,7 +1374,7 @@ static bool check_msg_counts(void)
 	if (!quiet)
 	    fprintf(stderr,
 		    "The messages sets contain %u non-spam and %u spam.  Bogotune "
-		    "requires at least %d non-spam and %d spam messages to run.\n",
+		    "requires at least %u non-spam and %u spam messages to run.\n",
 		    ns_cnt, sp_cnt, TEST_COUNT, TEST_COUNT);
 	exit(EX_ERROR);
     }
@@ -1359,6 +1382,15 @@ static bool check_msg_counts(void)
     return ok;
 }
 
+static void show_elapsed_time(int beg, int end, uint cnt, double val,
+			      const char *lbl1, const char *lbl2)
+{
+    int tm = end - beg;
+    if (!fMakeCheck)
+	printf("    %dm:%02ds for %u %s.  avg: %.1f %s\n",
+	       MIN(tm), SECONDS(tm), cnt, lbl1, val, lbl2);
+}
+
 static rc_t bogotune(void)
 {
     bool skip;
@@ -1370,11 +1402,12 @@ static rc_t bogotune(void)
 
     bogotune_init();
 
-    if (bogolex) {
-	if (check_msgcount_parms())
-	    load_wordlist(load_hook, ns_and_sp->train);
-	else
+    if (bogolex_file != NULL) {
+	if (!check_msgcount_parms())
 	    exit(EX_ERROR);
+	load_wordlist(load_hook, ns_and_sp->train);
+	read_mailbox(bogolex_file, NULL);
+	return status;
     }
 
     beg = time(NULL);
@@ -1385,12 +1418,6 @@ static rc_t bogotune(void)
     /* Note: memory usage highest while reading messages */
     /* usage decreases as distribute() converts to count format */
 
-    if (bogolex && bogolex_file != NULL) {
-	robx = ROBX;			/* needed for degen */
-	read_mailbox(bogolex_file, NULL);
-	return status;
-    }
-
     /* read all messages, merge training sets, look up scoring sets */
     ns_cnt = filelist_read(REG_GOOD, ham_files);
     sp_cnt = filelist_read(REG_SPAM, spam_files);
@@ -1401,9 +1428,6 @@ static rc_t bogotune(void)
 	show_elapsed_time(beg, end, ns_cnt + sp_cnt, (double)cnt/(end-beg), "messages", "msg/sec");
     }
 
-    if (bogolex)
-	return status;
-
     distribute(REG_GOOD, ns_msglists);
     distribute(REG_SPAM, sp_msglists);
 
@@ -1425,13 +1449,13 @@ static rc_t bogotune(void)
     sp_cnt = count_messages(sp_msglists);
     cnt = ns_cnt + sp_cnt;
 
-    if (ds_file && !check_msg_counts())
-	exit(EX_ERROR);
+    if (ds_flag == DS_DSK && !check_msg_counts())
+	exit(exit_zero ? EX_OK : EX_ERROR);
 
     fflush(stdout);
 
-    check_percent = CHECK_PCT;	/* for checking high scoring non-spam
-				** and low scoring spam */
+    check_percent = CHECK_PCT;	/* for checking low scoring spam
+				** and high scoring non-spam */
 
     ns_scores = xcalloc(ns_cnt, sizeof(double));
     sp_scores = xcalloc(sp_cnt, sizeof(double));
@@ -1450,10 +1474,7 @@ static rc_t bogotune(void)
     ** investigated to [x-0.1, x+0.1].
     */
 
-    if (user_robx > EPS)
-	robx = user_robx;
-    else if (ds_file != NULL)
-	robx = get_robx();
+    robx = get_robx();
 
     /*
     ** 6.  Calculate fp target
@@ -1613,11 +1634,21 @@ static rc_t bogotune(void)
 			print_sp_scores(fn-10, fn, 10);
 		    }
 #endif
+		    if (fMakeCheck && cnt >= cMakeCheck)
+			break;
 		}
+		if (fMakeCheck && cnt >= cMakeCheck)
+		    break;
 	      }
+	      if (fMakeCheck && cnt >= cMakeCheck)
+		  break;
 	    }
+	    if (fMakeCheck && cnt >= cMakeCheck)
+		break;
 	  }
 	  fflush(stdout);
+	  if (fMakeCheck && cnt >= cMakeCheck)
+	      break;
 	}
 
 	if (verbose >= TIME) {
@@ -1675,55 +1706,59 @@ int main(int argc, char **argv) /*@globals errno,stderr,stdout@*/
 {
     ex_t exitcode = EX_OK;
 
-    atexit(bt_exit);
-
     fBogotune = true;		/* for rob_compute_spamicity() */
 
-    dbgout = stdout;
-    debug_mask=MASK_BIT('R');
-    debug_mask=0;
+    dbgout = stderr;
 
     progtype = build_progtype(progname, DB_TYPE);
 
     ham_files  = filelist_new("ham");
     spam_files = filelist_new("spam");
 
-    /* directories from command line and config file are already handled */
-
-    ds_file = get_directory(PR_ENV_BOGO);
-    if (ds_file == NULL)
-	ds_file = get_directory(PR_ENV_HOME);
-
     /* process args and read mailboxes */
     process_arglist(argc, argv);
 
-    if (ds_file) {
-	set_bogohome(ds_file);
-	check_wordlist_path();
-    }
+    /* directories from command line and config file are already handled */
+    if (ds_flag == DS_DSK) {
 
-    ds_init();
+	bfpath *bfp;
+
+	if (ds_path == NULL)
+	    ds_path = get_directory(PR_ENV_BOGO);
+	if (ds_path == NULL)
+	    ds_path = get_directory(PR_ENV_HOME);
+
+	set_bogohome(ds_path);
+	bfp = bfpath_create(ds_path);
+
+	if (!bfpath_check_mode(bfp, BFP_MUST_EXIST)) {
+	    fprintf(stderr, "Can't open wordlist '%s'\n", bfp->filepath);
+	    exit(EX_ERROR);
+	}
+
+	if (bfp->exists && bfp->isdir) {
+	    bfpath_free(bfp);
+	    ds_path = mxcat(ds_path, DIRSEP_S, WORDLIST, NULL);	
+	    bfp = bfpath_create(ds_path);
+	    if (!bfpath_check_mode(bfp, BFP_MUST_EXIST)) {
+		fprintf(stderr, "Can't open wordlist '%s'\n", bfp->filepath);
+		exit(EX_ERROR);
+	    }
+	}
+
+	env = ds_init(bfp);
+	
+	init_wordlist("word", ds_path, 0, WL_REGULAR);
+    }
 
     bogotune();
 
     bogotune_free();
 
-    ds_cleanup();
+    if (ds_flag == DS_DSK)
+	ds_cleanup(env);
 
     exit(exitcode);
 }
 
-static void bt_exit(void)
-{
-    return;
-}
-
-static void show_elapsed_time(int beg, int end, uint cnt, double val,
-			      const char *lbl1, const char *lbl2)
-{
-    int tm = end - beg;
-    printf("    %dm:%02ds for %u %s.  avg: %.1f %s\n",
-	   MIN(tm), SECONDS(tm), cnt, lbl1, val, lbl2);
-}
-
 /* End */
