@@ -1,4 +1,4 @@
-/* $Id: charset.h,v 1.1 2003/02/03 16:55:14 relson Exp $ */
+/* $Id: charset.h,v 1.6 2005/01/09 17:32:34 relson Exp $ */
 
 /*****************************************************************************
 
@@ -19,6 +19,11 @@ extern bool replace_nonascii_characters;
 extern byte charset_table[256];
 extern byte casefold_table[256];
 extern void got_charset(const char *);
-extern void init_charset_table(const char *charset_name, bool use_default);
+extern void set_charset(const char *charset);
+extern void init_charset_table(const char *charset_name);
 
-#endif	/* CHARSET_H */
+#if	defined(CP866) && !defined(ENABLE_ICONV)
+extern int  decode_and_htmlUNICODE_to_cp866(byte *buf, int len);
+#endif
+
+#endif /* CHARSET_H */
