@@ -1,4 +1,4 @@
-/* $Id: datastore.c,v 1.45 2004/06/28 01:43:29 relson Exp $ */
+/* $Id: datastore.c,v 1.78 2005/04/04 11:16:20 relson Exp $ */
 
 /*****************************************************************************
 
@@ -14,15 +14,22 @@ David Relson <relson@osagesoftware.com>  2003
 
 #include "common.h"
 
+#include <assert.h>
 #include <string.h>
 #include <stdlib.h>
 #include <limits.h>
 
+#ifdef ENABLE_DB_DATASTORE
+#include <db.h>
+#endif
+
 #include "datastore.h"
 #include "datastore_db.h"
+#include "datastore_db_private.h"
 
 #include "error.h"
 #include "maint.h"
+#include "rand_sleep.h"
 #include "swap.h"
 #include "word.h"
 #include "xmalloc.h"
@@ -31,6 +38,34 @@ David Relson <relson@osagesoftware.com>  2003
 
 YYYYMMDD today;			/* date as YYYYMMDD */
 
+/* OO function list */
+
+static dsm_t dsm_dummies = {
+    /* public -- used in datastore.c */
+    NULL,	/* dsm_begin           */
+    NULL,	/* dsm_abort           */
+    NULL,	/* dsm_commit          */
+    /* private -- used in datastore_db_*.c */
+    NULL,	/* dsm_env_init         */
+    NULL,	/* dsm_cleanup          */
+    NULL,	/* dsm_cleanup_lite     */
+    NULL,	/* dsm_get_env_dbe      */
+    NULL,	/* dsm_database_name    */
+    NULL,	/* dsm_recover_open     */
+    NULL,	/* dsm_auto_commit_flags*/                    
+    NULL,	/* dsm_get_rmw_flag     */
+    NULL,	/* dsm_lock             */
+    NULL,	/* dsm_common_close     */
+    NULL,	/* dsm_sync             */
+    NULL,	/* dsm_log_flush        */
+    NULL,	/* dsm_recover          */
+    NULL,	/* dsm_remove           */
+    NULL,	/* dsm_checkpoint       */
+    NULL,	/* dsm_purgelogs        */
+    NULL,	/* dsm_verify           */
+    NULL	/* dsm_pagesize         */
+};
+
 /* Function definitions */
 
 static
@@ -101,8 +136,7 @@ static void convert_internal_to_external(dsh_t *dsh, dsv_t *in_data, dbv_t *ex_d
     return;
 }
 
-dsh_t *dsh_init(
-    void *dbh)			/* database handle from db_open() */
+dsh_t *dsh_init(void *dbh)		/* database handle from db_open() */
 {
     dsh_t *val = xmalloc(sizeof(*val));
     val->dbh = dbh;
@@ -117,29 +151,32 @@ void dsh_free(void *vhandle)
     return;
 }
 
-void *ds_open(const char *path, const char *name, dbmode_t open_mode)
+void *ds_open(void *dbe, bfpath *bfp, dbmode_t open_mode)
 {
     dsh_t *dsh;
     void *v;
 
-    v = db_open(path, name, open_mode);
+    v = db_open(dbe, bfp, open_mode); /* FIXME */
 
     if (!v)
 	return NULL;
 
-    ds_init();
     dsh = dsh_init(v);
 
-    if (db_created(v) && ! (open_mode & DS_LOAD))
+    if (db_created(v) && ! (open_mode & DS_LOAD)) {
+	if (DST_OK != ds_txn_begin(dsh))
+	    exit(EX_ERROR);
 	ds_set_wordlist_version(dsh, NULL);
-
+	if (ds_txn_commit(dsh))
+	    exit(EX_ERROR);
+    }
     return dsh;
 }
 
-void ds_close(/*@only@*/ void *vhandle, bool nosync  /** Normally false, if true, do not synchronize data. This should not be used in regular operation but only to ease the disk I/O load when the lock operation failed. */)
+void ds_close(/*@only@*/ void *vhandle)
 {
     dsh_t *dsh = vhandle;
-    db_close(dsh->dbh, nosync);
+    db_close(dsh->dbh);
     xfree(dsh);
 }
 
@@ -182,19 +219,26 @@ int ds_read(void *vhandle, const word_t *word, /*@out@*/ dsv_t *val)
 
 	if (DEBUG_DATABASE(3)) {
 	    fprintf(dbgout, "ds_read: [%.*s] -- %lu,%lu\n",
-		    CLAMP_INT_MAX(word->leng), word->text,
+		    CLAMP_INT_MAX(word->leng), (const char *)word->text,
 		    (unsigned long)val->spamcount,
 		    (unsigned long)val->goodcount);
 	}
-	break;
+	return 0;
 
     case DS_NOTFOUND:
 	if (DEBUG_DATABASE(3)) {
 	    fprintf(dbgout, "ds_read: [%.*s] not found\n", 
 		    CLAMP_INT_MAX(word->leng), (char *) word->text);
 	}
+	return 1;
+
+    case DS_ABORT_RETRY:
+	if (DEBUG_DATABASE(1)) {
+	    print_error(__FILE__, __LINE__, "ds_read('%.*s') was aborted to recover from a deadlock.",
+		    CLAMP_INT_MAX(word->leng), (char *) word->text);
+	}
 	break;
-	    
+
     default:
 	fprintf(dbgout, "ret=%d, DS_NOTFOUND=%d\n", ret, DS_NOTFOUND);
 	print_error(__FILE__, __LINE__, "ds_read( '%.*s' ), err: %d, %s", 
@@ -202,7 +246,7 @@ int ds_read(void *vhandle, const word_t *word, /*@out@*/ dsv_t *val)
 	exit(EX_ERROR);
     }
 
-    return found ? 0 : 1;
+    return ret;
 }
 
 int ds_write(void *vhandle, const word_t *word, dsv_t *val)
@@ -231,7 +275,7 @@ int ds_write(void *vhandle, const word_t *word, dsv_t *val)
 
     if (DEBUG_DATABASE(3)) {
 	fprintf(dbgout, "ds_write: [%.*s] -- %lu,%lu,%lu\n",
-		CLAMP_INT_MAX(word->leng), word->text,
+		CLAMP_INT_MAX(word->leng), (const char *)word->text,
 		(unsigned long)val->spamcount,
 		(unsigned long)val->goodcount,
 		(unsigned long)val->date);
@@ -255,17 +299,41 @@ int ds_delete(void *vhandle, const word_t *word)
     return ret;		/* 0 if ok */
 }
 
+int ds_txn_begin(void *vhandle) {
+    dsh_t *dsh = vhandle;
+    if (dsm->dsm_begin == NULL)
+	return 0;
+    else
+	return dsm->dsm_begin(dsh->dbh);
+}
+
+int ds_txn_abort(void *vhandle) {
+    dsh_t *dsh = vhandle;
+    if (dsm->dsm_abort == NULL)
+	return 0;
+    else
+	return dsm->dsm_abort(dsh->dbh);
+}
+
+int ds_txn_commit(void *vhandle) {
+    dsh_t *dsh = vhandle;
+    if (dsm->dsm_commit == NULL)
+	return 0;
+    else
+	return dsm->dsm_commit(dsh->dbh);
+}
+
 typedef struct {
     ds_foreach_t *hook;
     dsh_t	 *dsh;
     void         *data;
 } ds_userdata_t;
 
-static int ds_hook(dbv_t *ex_key,
-		   dbv_t *ex_data,
-		   void *userdata)
+static ex_t ds_hook(dbv_t *ex_key,
+		    dbv_t *ex_data,
+		    void *userdata)
 {
-    int ret;
+    ex_t ret;
     word_t w_key;
     dsv_t in_data;
     ds_userdata_t *ds_data = userdata;
@@ -279,13 +347,13 @@ static int ds_hook(dbv_t *ex_key,
 
     ret = (*ds_data->hook)(&w_key, &in_data, ds_data->data);
 
-    return ret;		/* 0 if ok */
+    return ret;		/* EX_OK if ok */
 }
 
-int ds_foreach(void *vhandle, ds_foreach_t *hook, void *userdata)
+ex_t ds_foreach(void *vhandle, ds_foreach_t *hook, void *userdata)
 {
     dsh_t *dsh = vhandle;
-    int ret;
+    ex_t ret;
     ds_userdata_t ds_data;
     ds_data.hook = hook;
     ds_data.dsh  = dsh;
@@ -298,22 +366,30 @@ int ds_foreach(void *vhandle, ds_foreach_t *hook, void *userdata)
 
 /* Wrapper for ds_foreach that opens and closes file */
 
-int ds_oper(const char *path, dbmode_t open_mode, 
-	    ds_foreach_t *hook, void *userdata)
+ex_t ds_oper(void *env, bfpath *bfp, dbmode_t open_mode, 
+	     ds_foreach_t *hook, void *userdata)
 {
-    int  ret = 0;
+    ex_t ret = EX_OK;
     void *dsh;
-
-    dsh = ds_open(CURDIR_S, path, open_mode);
+    
+    dsh = ds_open(env, bfp, open_mode);
 
     if (dsh == NULL) {
-	fprintf(stderr, "Can't open file '%s'\n", path);
+	fprintf(stderr, "Can't open file '%s'\n", bfp->filepath);
 	exit(EX_ERROR);
     }
 
-    ret = ds_foreach(dsh, hook, userdata);
+    if (DST_OK == ds_txn_begin(dsh)) {
+	if (ret == EX_OK)
+	    ret = ds_foreach(dsh, hook, userdata);
 
-    ds_close(dsh, false);
+	if (ret != EX_OK) { ds_txn_abort(dsh); }
+	else
+	    if (ds_txn_commit(dsh) != DST_OK)
+		ret = EX_ERROR;
+    }
+
+    ds_close(dsh);
 
     return ret;
 }
@@ -321,21 +397,30 @@ int ds_oper(const char *path, dbmode_t open_mode,
 static word_t  *msg_count_tok;
 static word_t  *wordlist_version_tok;
 
-void ds_init()
+void *ds_init(bfpath *bfp)
 {
-    db_init();
+    void *dbe;
+
+    dbe = dbe_init(bfp);
+
+    if (dsm == NULL)
+	dsm = &dsm_dummies;
+
     if (msg_count_tok == NULL) {
-	msg_count_tok = word_new((const byte *)MSG_COUNT, strlen(MSG_COUNT));
+	msg_count_tok = word_news(MSG_COUNT);
     }
     if (wordlist_version_tok == NULL) {
-	wordlist_version_tok = word_new((const byte *)WORDLIST_VERSION, strlen(WORDLIST_VERSION));
+	wordlist_version_tok = word_news(WORDLIST_VERSION);
     }
+
+    return dbe;
 }
 
 /* Cleanup storage allocation */
-void ds_cleanup()
+void ds_cleanup(void *dbe)
 {
-    db_cleanup();
+    if (dsm->dsm_cleanup != NULL)
+	dsm->dsm_cleanup(dbe);
     xfree(msg_count_tok);
     xfree(wordlist_version_tok);
     msg_count_tok = NULL;
@@ -345,47 +430,45 @@ void ds_cleanup()
 /*
   Get the number of messages associated with database.
 */
-bool ds_get_msgcounts(void *vhandle, dsv_t *val)
+int ds_get_msgcounts(void *vhandle, dsv_t *val)
 {
-    int rc;
     dsh_t *dsh = vhandle;
 
-    rc = ds_read(dsh, msg_count_tok, val);
-
-    return rc == 0;
+    return ds_read(dsh, msg_count_tok, val);
 }
 
 /*
  Set the number of messages associated with database.
 */
-void ds_set_msgcounts(void *vhandle, dsv_t *val)
+int  ds_set_msgcounts(void *vhandle, dsv_t *val)
 {
     dsh_t *dsh = vhandle;
 
     val->date = today;
 
-    ds_write(dsh, msg_count_tok, val);
+    return ds_write(dsh, msg_count_tok, val);
+}
 
-    return;
+void *ds_get_dbenv(void *vhandle)
+{
+    dsh_t *dsh = vhandle;
+    return db_get_env(dsh->dbh);
 }
 
 /*
   Get the wordlist version associated with database.
 */
-bool ds_get_wordlist_version(void *vhandle, dsv_t *val)
+int ds_get_wordlist_version(void *vhandle, dsv_t *val)
 {
-    int rc;
     dsh_t *dsh = vhandle;
 
-    rc = ds_read(dsh, wordlist_version_tok, val);
-
-    return rc == 0;
+    return ds_read(dsh, wordlist_version_tok, val);
 }
 
 /*
  Set the wordlist version associated with database.
 */
-void ds_set_wordlist_version(void *vhandle, dsv_t *val)
+int ds_set_wordlist_version(void *vhandle, dsv_t *val)
 {
     dsh_t *dsh = vhandle;
     dsv_t  tmp;
@@ -399,12 +482,58 @@ void ds_set_wordlist_version(void *vhandle, dsv_t *val)
 
     val->date = today;
 
-    ds_write(dsh, wordlist_version_tok, val);
-
-    return;
+    return ds_write(dsh, wordlist_version_tok, val);
 }
 
 const char *ds_version_str(void)
 {
     return db_version_str();
 }
+
+ex_t ds_recover(bfpath *bfp, bool catastrophic)
+{
+    if (dsm->dsm_recover == NULL)
+	return EX_OK;
+    else
+	return dsm->dsm_recover(bfp, catastrophic, true);
+}
+
+ex_t ds_remove(bfpath *bfp)
+{
+    if (dsm->dsm_remove == NULL)
+	return EX_OK;
+    else
+	return dsm->dsm_remove(bfp);
+}
+
+ex_t ds_checkpoint(bfpath *bfp)
+{
+    if (dsm->dsm_checkpoint == NULL)
+	return EX_OK;
+    else
+	return dsm->dsm_checkpoint(bfp);
+}
+
+ex_t ds_purgelogs(bfpath *bfp)
+{
+    if (dsm->dsm_purgelogs == NULL)
+	return EX_OK;
+    else
+	return dsm->dsm_purgelogs(bfp);
+}
+
+ex_t ds_verify(bfpath *bfp)
+{
+    if (dsm->dsm_verify == NULL)
+	return EX_OK;
+    else
+	return dsm->dsm_verify(bfp);
+}
+
+u_int32_t ds_pagesize(bfpath *bfp)
+{
+    if (dsm->dsm_pagesize == NULL)
+	return 0;
+    else
+	return dsm->dsm_pagesize(bfp);
+}
