@@ -1,4 +1,4 @@
-/* $Id: datastore_tdb.c,v 1.30 2004/06/28 01:43:30 relson Exp $ */
+/* $Id: datastore_tdb.c,v 1.51 2005/04/04 11:16:22 relson Exp $ */
 
 /*****************************************************************************
 
@@ -33,11 +33,6 @@ typedef struct {
     TDB_CONTEXT *dbp;
 } dbh_t;
 
-/* dummy infrastructure, to be expanded by environment
- * or transactional initialization/shutdown */
-
-static bool init = false;
-
 /* Function definitions */
 
 const char *db_version_str(void)
@@ -45,18 +40,14 @@ const char *db_version_str(void)
     return "TrivialDB";
 }
 
-static dbh_t *dbh_init(const char *path, const char *name)
+static dbh_t *dbh_init(bfpath *bfp)
 {
     dbh_t *handle;
-    size_t len = strlen(path) + strlen(name) + 2;
 
     handle = xmalloc(sizeof(dbh_t));
     memset(handle, 0, sizeof(dbh_t));	/* valgrind */
 
-    handle->path = xstrdup(path);
-
-    handle->name = xmalloc(len);
-    build_path(handle->name, len, path, name);
+    handle->name = xstrdup(bfp->filepath);
 
     handle->locked  = false;
     handle->created = false;
@@ -96,7 +87,7 @@ bool db_created(void *vhandle)
   Initialize database.
   Returns: pointer to database handle on success, NULL otherwise.
 */
-void *db_open(const char *db_file, const char *name, dbmode_t open_mode)
+void *db_open(void *dummy, bfpath *bfp, dbmode_t open_mode)
 {
     dbh_t *handle;
 
@@ -104,6 +95,8 @@ void *db_open(const char *db_file, const char *name, dbmode_t open_mode)
     int open_flags;
     TDB_CONTEXT *dbp;
 
+    (void)dummy;
+
     if (open_mode & DS_WRITE) {
 	open_flags = O_RDWR;
 	tdb_flags = 0;
@@ -113,14 +106,14 @@ void *db_open(const char *db_file, const char *name, dbmode_t open_mode)
     	tdb_flags = TDB_NOLOCK;
     }
 
-    handle = dbh_init(db_file, name);
+    handle = dbh_init(bfp);
 
     if (handle == NULL) return NULL;
 
-    dbp = handle->dbp = tdb_open(handle->name, 0, tdb_flags, open_flags, 0664);
+    dbp = handle->dbp = tdb_open(handle->name, 0, tdb_flags, open_flags, DS_MODE);
 
     if ((dbp == NULL) && (open_mode & DS_WRITE)) {
-	dbp = handle->dbp = tdb_open(handle->name, 0, tdb_flags, open_flags | O_CREAT, 0664);
+	dbp = handle->dbp = tdb_open(handle->name, 0, tdb_flags, open_flags | O_CREAT, DS_MODE);
 	if (dbp != NULL)
 	    handle->created = true;
     }
@@ -199,14 +192,12 @@ int db_get_dbvalue(void *vhandle, const dbv_t *token, /*@out@*/ dbv_t *val)
 
     val->leng = db_data.dsize;		/* read count */
     memcpy(val->data, db_data.dptr, db_data.dsize);
-    
-    free(db_data.dptr);
 
     return 0;
 }
 
 
-int db_set_dbvalue(void *vhandle, const dbv_t *token, dbv_t *val)
+int db_set_dbvalue(void *vhandle, const dbv_t *token, const dbv_t *val)
 {
     int ret;
     TDB_DATA db_key;
@@ -234,7 +225,7 @@ int db_set_dbvalue(void *vhandle, const dbv_t *token, dbv_t *val)
 
 
 /* Close files and clean up. */
-void db_close(void *vhandle, bool nosync)
+void db_close(void *vhandle)
 {
     int ret;
     dbh_t *handle = vhandle;
@@ -242,7 +233,7 @@ void db_close(void *vhandle, bool nosync)
     if (handle == NULL) return;
 
     if (DEBUG_DATABASE(1))
-	fprintf(dbgout, "db_close (%s) %s\n", handle->name, nosync ? "nosync" : "sync");
+	fprintf(dbgout, "db_close (%s)\n", handle->name);
 
     if (handle->locked) {
 	tdb_unlockall(handle->dbp);
@@ -297,7 +288,7 @@ static int tdb_traversor(/*@unused@*/ __attribute__ ((unused)) TDB_CONTEXT * tdb
 }
 
 
-int db_foreach(void *vhandle, db_foreach_t hook, void *userdata)
+ex_t db_foreach(void *vhandle, db_foreach_t hook, void *userdata)
 {
     int ret;
     dbh_t *handle = vhandle;
@@ -315,7 +306,7 @@ int db_foreach(void *vhandle, db_foreach_t hook, void *userdata)
 	exit(EX_ERROR);
     }
 
-    return 0;
+    return EX_OK;
 }
 
 /*
@@ -331,15 +322,18 @@ int db_foreach(void *vhandle, db_foreach_t hook, void *userdata)
  */
 
 static struct tdb_errname {
-	enum TDB_ERROR ecode; const char *estring;
-} emap[] = { {TDB_SUCCESS, "Success"},
-	     {TDB_ERR_CORRUPT, "Corrupt database"},
-	     {TDB_ERR_IO, "IO Error"},
-	     {TDB_ERR_LOCK, "Locking error"},
-	     {TDB_ERR_OOM, "Out of memory"},
-	     {TDB_ERR_EXISTS, "Record exists"},
-	     {TDB_ERR_NOLOCK, "Lock exists on other keys"},
-	     {TDB_ERR_NOEXIST, "Record does not exist"} };
+    enum TDB_ERROR ecode; 
+    const char *estring;
+} emap[] = { 
+    {TDB_SUCCESS,     "Success"},
+    {TDB_ERR_CORRUPT, "Corrupt database"},
+    {TDB_ERR_IO,      "IO Error"},
+    {TDB_ERR_LOCK,    "Locking error"},
+    {TDB_ERR_OOM,     "Out of memory"},
+    {TDB_ERR_EXISTS,  "Record exists"},
+    {TDB_ERR_NOLOCK,  "Lock exists on other keys"},
+    {TDB_ERR_NOEXIST, "Record does not exist"}
+};
 
 /* Error string for the last tdb error */
 const char *db_str_err(int j)
@@ -352,14 +346,3 @@ const char *db_str_err(int j)
 			return emap[i].estring;
 	return "Invalid error code";
 }
-
-int db_init(void)
-{
-    init = true; 
-    return 0;
-}
-
-void db_cleanup(void)
-{
-    init = false;
-}
