@@ -1,4 +1,4 @@
-/* $Id: globals.c,v 1.62 2004/08/22 15:25:04 relson Exp $ */
+/* $Id: globals.c,v 1.73 2005/04/04 11:16:22 relson Exp $ */
 
 /*****************************************************************************
 
@@ -22,7 +22,6 @@ bulk_t	bulk_mode = B_NORMAL;		/* '-b, -B' */
 bool	suppress_config_file;		/* '-C' */
 bool	nonspam_exits_zero;		/* '-e' */
 FILE	*fpin = NULL;			/* '-I' */
-bool	fisher;				/* '-f' */
 bool	logflag;			/* '-l' */
 bool	mbox_mode;			/* '-M' */
 bool	replace_nonascii_characters;	/* '-n' */
@@ -35,7 +34,6 @@ int	bogotest;			/* '-X', env("BOGOTEST") */
 int	verbose;			/* '-v' */
 
 /* config file options */
-int	max_repeats=1;			/* set default value */
 double	min_dev;
 double	ham_cutoff = HAM_CUTOFF;
 double	spam_cutoff;
@@ -48,8 +46,15 @@ const char	*stats_prefix;
 /* for lexer_v3.l */
 bool	header_line_markup = true;	/* -H */
 
+/* for  transactions */
+#ifndef	ENABLE_TRANSACTIONS
+bool fTransaction = false;
+#else
+bool fTransaction = true;
+#endif
+
 /* for  bogotune */
-bool fBogotune = false;
+bool 	fBogotune = false;
 
 /* for  bogoconfig.c, prob.c, rstats.c and score.c */
 double	robx = 0.0;
