@@ -1,4 +1,4 @@
-/* $Id: globals.h,v 1.69 2004/08/22 15:25:19 relson Exp $ */
+/* $Id: globals.h,v 1.77 2005/03/09 01:14:46 relson Exp $ */
 
 #ifndef GLOBALS_H
 #define GLOBALS_H
@@ -32,7 +32,6 @@ extern	int	verbose;		/* '-v' */
 extern	bool	replace_nonascii_characters;	/* '-n' */
 
 /* config file options */
-extern	int	max_repeats;
 extern	double	min_dev;
 extern	double	ham_cutoff;
 extern	double	spam_cutoff;
@@ -53,8 +52,8 @@ extern	const char *const system_config_file;
 
 /* for msgcounts.c */
 #define	MSG_COUNT	".MSG_COUNT"
-extern	double		 msgs_good;
-extern	double		 msgs_bad;
+extern	uint		 msgs_good;
+extern	uint		 msgs_bad;
 
 /* for passthrough.c */
 extern size_t msg_register_size;
@@ -79,7 +78,7 @@ extern bool onlyprint;
 #define	SP_ESF		1.0
 #define	NS_ESF		1.0
 
-#define HAM_CUTOFF	0.00	/* 0.00 for two-state, 0.45 for three-state */
+#define HAM_CUTOFF	0.45	/* 0.45 for three-state, 0.00 for two-state  */
 #define SPAM_CUTOFF	0.99
 
 #define ROBX_W		".ROBX"
@@ -96,6 +95,9 @@ extern	bool fBogotune;
 #define	DB_CACHESIZE	4	/* in MB */
 extern	uint	db_cachesize;
 
+/* for  transactions */
+extern	bool	fTransaction;
+
 /* other */
 
 extern FILE  *fpo;
@@ -109,4 +111,6 @@ extern	bool	unsure_stats;	/* true if print stats for unsures */
 #define	WORDLIST_VERSION	".WORDLIST_VERSION"
 extern	uint	wordlist_version;
 
+extern	bool	fDie;		/* true if SIGINT or SIGTERM received */
+
 #endif
