@@ -1,11 +1,11 @@
-/* $Id: lexer.c,v 1.103 2004/10/13 23:18:38 relson Exp $ */
+/* $Id: lexer.c,v 1.118 2005/03/24 00:01:49 m-a Exp $ */
 
-/*
- * NAME
- *   lexer.c -- bogofilter's lexical analyzer (control routines)
+/**
+ * \file lexer.c
+ * bogofilter's lexical analyzer (control routines)
  *
- *   01/01/2003 - split out of lexer.l
-*/
+ * \date 2003-01-01 split out of lexer.l
+ */
 
 #include "common.h"
 
@@ -17,6 +17,7 @@
 #include "bogoreader.h"
 #include "charset.h"
 #include "error.h"
+#include "iconvert.h"
 #include "lexer.h"
 #include "memstr.h"
 #include "mime.h"
@@ -61,7 +62,7 @@ void lexer_init(void)
     mime_reset();
     token_init();
     lexer_v3_init(NULL);
-    init_charset_table(charset_default, true);
+    init_charset_table(charset_default);
 }
 
 static void lexer_display_buffer(buff_t *buff)
@@ -74,18 +75,22 @@ static void lexer_display_buffer(buff_t *buff)
 	fputc('\n', dbgout);
 }
 
-/* Check for lines wholly composed of printable characters as they can cause a scanner abort 
-   "input buffer overflow, can't enlarge buffer because scanner uses REJECT"
-*/
-static bool not_long_token(byte *buf, uint count)
+/**
+ * Check for lines wholly composed of printable characters as they can
+ * cause a scanner abort "input buffer overflow, can't enlarge buffer
+ * because scanner uses REJECT"
+ *
+ * \bug this function must go, we need to fix the lexer
+ */
+static bool long_token(byte *buf, uint count)
 {
     uint i;
     for (i=0; i < count; i += 1) {
 	byte c = buf[i];
 	if ((iscntrl(c) || isspace(c) || ispunct(c)) && (c != '_'))
-	    return true;
+	    return false;
     }
-    return false;
+    return true;
 }
 
 static int yy_get_new_line(buff_t *buff)
@@ -122,35 +127,65 @@ static int yy_get_new_line(buff_t *buff)
 
     if (count >= 0 && DEBUG_LEXER(0))
 	lexer_display_buffer(buff);
-    
+
     /* skip spam_header ("X-Bogosity:") lines */
     while (msg_header
 	   && count != EOF
-	   /* don't skip if inside message/rfc822 */
+/* don't skip if inside message/rfc822 */
 	   && msg_state == msg_state->parent
 	   && memcmp(buff->t.text,spam_header_name,hdrlen) == 0) {
 	count = skip_folded_line(buff);
     }
 
-    /* Also, save the text on a linked list of lines.
-     * Note that we store fixed-length blocks here, not lines.
-     * One very long physical line could break up into more
-     * than one of these. */
-
-    if (passthrough && passmode == PASS_MEM && count > 0)
-	textblock_add(buff->t.text+buff->read, (size_t) count);
-
     return count;
 }
 
 static int get_decoded_line(buff_t *buff)
 {
+    int count;
+    static buff_t *temp = NULL;
+
     uint used = buff->t.leng;
     byte *buf = buff->t.text + used;
-    int count = yy_get_new_line(buff);
+
+#ifndef	ENABLE_ICONV
+    temp = buff;
+#else
+    if (temp == NULL)
+	temp = (buff_t *) calloc(sizeof(buff_t), 1);
+    /* UTF-8 uses up to six octets per character, make output buffer
+     * large enough */
+    if (temp->size < buff->size * 6) {
+	xfree(temp->t.text);
+	temp->size = buff->size * 6;
+	temp->t.text = (byte *) xmalloc(temp->size + D);
+    }
+
+    temp->t.leng = temp->read = 0;
+#endif
+
+    count = yy_get_new_line(temp);
+
+    /* Save the text on a linked list of lines.
+     * Note that we store fixed-length blocks here, not lines.
+     * One very long physical line could break up into more
+     * than one of these. */
+
+    if (passthrough && passmode == PASS_MEM && count > 0)
+	textblock_add(temp->t.text+temp->read, (size_t) count);
+
+#ifdef	ENABLE_ICONV
+    iconvert(temp, buff);
+    /*
+     * iconvert, treating multi-byte sequences, can shrink or enlarge
+     * the output compared to its input.  Correct count.
+     */
+    if (count > 0)
+	count = buff->t.leng;
+#endif
 
     if (count == EOF) {
-	if ( !ferror(fpin)) 
+	if ( !ferror(fpin))
 	    return YY_NULL;
 	else {
 	    print_error(__FILE__, __LINE__, "input in flex scanner failed\n");
@@ -169,7 +204,8 @@ static int get_decoded_line(buff_t *buff)
     {
 	word_t line;
 	uint decoded_count;
-        line.leng = (uint) (buff->t.leng - used);
+
+	line.leng = (uint) (buff->t.leng - used);
 	line.text = buff->t.text + used;
 
 	decoded_count = mime_decode(&line);
@@ -177,7 +213,7 @@ static int get_decoded_line(buff_t *buff)
 	if (decoded_count != 0 && decoded_count < (uint) count) {
 	    buff->t.leng -= (uint) (count - decoded_count);
 	    count = (int) decoded_count;
-	    if (DEBUG_LEXER(1)) 
+	    if (DEBUG_LEXER(1))
 		lexer_display_buffer(buff);
 	}
     }
@@ -188,8 +224,8 @@ static int get_decoded_line(buff_t *buff)
 	*(buf + count - 1) = (byte) '\n';
     }
 
-    if (buff->t.leng < buff->size)	/* for easier debugging - removable */
-	Z(buff->t.text[buff->t.leng]);	/* for easier debugging - removable */
+    if (buff->t.leng < buff->size)     /* for easier debugging - removable */
+	Z(buff->t.text[buff->t.leng]);  /* for easier debugging - removable */
 
     return count;
 }
@@ -201,14 +237,15 @@ static int skip_folded_line(buff_t *buff)
 	buff->t.leng = 0;
 	count = reader_getline(buff);
 	yylineno += 1;
-	if (!isspace(buff->t.text[0])) 
+	/* only check for LWSP-char (RFC-822) aka. WSP (RFC-2822),
+	 * these only include SP and HTAB */
+	if (buff->t.text[0] != ' '
+		&& buff->t.text[0] != '\t')
 	    return count;
 	/* Check for empty line which terminates message header */
 	if (is_eol((char *)buff->t.text, count))
 	    return count;
     }
-
-/*    return EOF; */
 }
 
 int buff_fill(buff_t *buff, size_t used, size_t need)
@@ -255,6 +292,8 @@ int yyinput(byte *buf, size_t used, size_t size)
 
     while ((cnt = get_decoded_line(&buff)) != 0) {
 
+	count += cnt;
+
 	/* Note: some malformed messages can cause xfgetsl() to report
 	** "Invalid buffer size, exiting."  ** and then abort.  This
 	** can happen when the parser is in html mode and there's a
@@ -263,31 +302,41 @@ int yyinput(byte *buf, size_t used, size_t size)
 	** The "fix" is to check for a nearly full lexer buffer and
 	** discard most of it.
 	*/
-	bool nearly_full = used > 1000 && used > size * 10;
 
-	count += cnt;
+	/* if not nearly full */
+	if (used < 1000 || used < size * 10)
+	    break;
 
-	if ((count <= (MAXTOKENLEN * 3 / 2)) || not_long_token(buff.t.text, (uint) count))
-	    if (!nearly_full)
-		break;
-
-	if (count >= MAXTOKENLEN * 2) {
-	    size_t shift = count - MAXTOKENLEN;
-	    memmove(buff.t.text, buff.t.text + shift, MAXTOKENLEN+D);
-	    count = MAXTOKENLEN;
-	    buff.t.leng = MAXTOKENLEN;
+	if (count >= MAXTOKENLEN * 2 && 
+	    long_token(buff.t.text, (uint) count)) {
+	    uint start = buff.t.leng - count;
+	    uint length = count - MAXTOKENLEN;
+	    buff_shift(&buff, start, length);
+	    count = buff.t.leng;
 	}
-
-	if (nearly_full)
+	else
 	    break;
     }
 
+    if (msg_state &&
+	msg_state->mime_disposition &&
+	(msg_state->mime_type == MIME_APPLICATION ||  
+	 msg_state->mime_type == MIME_IMAGE)) {
+	return (count == EOF ? 0 : count);   /* not decode at all */
+    }
+
+/* EK -  decoding things like &#1084 and charset_table */
+#ifdef	CP866
+    count = htmlUNICODE_decode(buf, count);
+#else
+#ifndef ENABLE_ICONV
     for (i = 0; i < count; i++ )
     {
 	byte ch = buf[i];
 	buf[i] = charset_table[ch];
     }
-
+#endif
+#endif
     return (count == EOF ? 0 : count);
 }
 
@@ -325,7 +374,7 @@ size_t text_decode(word_t *w)
 	switch (tolower(typ[1])) {		/* ... encoding type */
 	case 'b':
 	    if (base64_validate(&n))
-		len = base64_decode(&n);	/* decode base64 */
+		len = base64_decode(&n);		/* decode base64 */
 	    break;
 	case 'q':
 	    if (qp_validate(&n, RFC2047))
@@ -357,7 +406,7 @@ size_t text_decode(word_t *w)
 	if (adjacent) {
 	    tmp = txt;
 	    while (adjacent && tmp < end) {
-		if (isspace(*tmp))
+		if (*tmp && strchr(" \t\r\n", *tmp))
 		    tmp += 1;
 		else
 		    adjacent = false;
