@@ -1,4 +1,4 @@
-/* $Id: maint.h,v 1.6 2004/05/02 16:37:09 relson Exp $ */
+/* $Id: maint.h,v 1.8 2005/04/04 11:16:22 relson Exp $ */
 
 #ifndef MAINT_H
 #define MAINT_H
@@ -21,7 +21,7 @@ extern	bool     upgrade_wordlist_version;
 
 /* Function Prototypes */
 void maintain_wordlists(void);
-int  maintain_wordlist_file(const char *db_file);
+ex_t maintain_wordlist_file(bfpath *bfp);
 
 bool discard_token(word_t *token, dsv_t *info);
 
