@@ -1,4 +1,4 @@
-/* $Id: robx.c,v 1.10 2004/09/05 12:33:31 relson Exp $ */
+/* $Id: robx.c,v 1.24 2005/04/04 11:16:22 relson Exp $ */
 
 /*****************************************************************************
 
@@ -24,6 +24,8 @@ AUTHOR:
 typedef struct robhook_data {
     double   sum;
     uint32_t count;
+    uint32_t spam_cnt;
+    uint32_t good_cnt;
     dsh_t    *dsh;
     double   scalefactor;
 } rhd_t;
@@ -58,61 +60,47 @@ static int robx_hook(word_t *key, dsv_t *data,
     struct robhook_data *rh = userdata;
 
     /* ignore system meta-data */
-    if (*key->text == '.')
-	return 0;
+    if (*key->text != '.')
+	robx_accum(rh, key, data);
 
-    robx_accum(rh, key, data);
-    
     return 0;
 }
 
-static double compute_robx(dsh_t *dsh)
+/** returns negative for failure.
+ * used by bogoutil and bogotune */
+double compute_robinson_x(void)
 {
+    int ret;
     double rx;
+    dsh_t *dsh;
+    wordlist_t *wordlist;
 
-    dsv_t val;
-    bool ok;
-    uint32_t good_cnt, spam_cnt;
     struct robhook_data rh;
 
-    ok = ds_get_msgcounts(dsh, &val);
+    open_wordlists(DS_READ);
+    wordlist = get_default_wordlist(word_lists);
 
-    if (!ok) {
-	fprintf(stderr, "Can't find message counts.\n");
-	exit(EX_ERROR);
-    }
+    dsh = wordlist->dsh;
 
-    spam_cnt = val.spamcount;
-    good_cnt = val.goodcount;
+    rh.spam_cnt = wordlist->msgcount[IX_SPAM];
+    rh.good_cnt = wordlist->msgcount[IX_GOOD];
+    rh.scalefactor = (double)rh.spam_cnt/(double)rh.good_cnt;
 
-    rh.scalefactor = (double)spam_cnt/(double)good_cnt;
     rh.dsh = dsh;
     rh.sum = 0.0;
     rh.count = 0;
 
-    ds_foreach(dsh, robx_hook, &rh);
+    ret = ds_foreach(dsh, robx_hook, &rh);
 
     rx = rh.sum/rh.count;
+    if (rh.count == 0)
+	ret = -1;
     if (verbose > 2)
-	printf("%s: %lu, %lu, scale: %f, sum: %f, cnt: %6d, .ROBX: %f\n",
-	       MSG_COUNT,
-	       (unsigned long)spam_cnt, (unsigned long)good_cnt,
+	printf("%s: %u, %u, scale: %f, sum: %f, cnt: %6d, .ROBX: %f\n",
+	       MSG_COUNT, rh.spam_cnt, rh.good_cnt,
 	       rh.scalefactor, rh.sum, (int)rh.count, rx);
 
-    return rx;
-}
+    close_wordlists(true);
 
-double compute_robinson_x(char *path)
-{
-    double rx;
-
-    set_wordlist_dir(path, PR_NONE);
-    open_wordlists(DS_READ);
-
-    rx = compute_robx(default_wordlist()->dsh);
-
-    close_wordlists(false);
-    free_wordlists();
-
-    return rx;
+    return ret ? -1 : rx;
 }
