@@ -1,4 +1,4 @@
-/* $Id: score.c,v 1.25 2004/10/19 23:49:47 relson Exp $ */
+/* $Id: score.c,v 1.53 2005/03/16 00:10:20 m-a Exp $ */
 
 /*****************************************************************************
 
@@ -19,6 +19,7 @@ NAME:
 #include "datastore.h"
 #include "msgcounts.h"
 #include "prob.h"
+#include "rand_sleep.h"
 #include "rstats.h"
 #include "score.h"
 #include "wordhash.h"
@@ -76,20 +77,9 @@ void msg_print_stats(FILE *fp)
 	rstats_print(unsure);
 }
 
-static void wordprob_add(wordcnts_t *cnts, uint count, uint idx)
-{
-    if (idx == IX_SPAM)
-	cnts->bad += count;
-    else
-	cnts->good += count;
-}
-
-static double wordprob_result(wordcnts_t *cnt)
-{
-    double fw = calc_prob(cnt->good, cnt->bad);
-    return fw;
-}
-
+/** search token in all lists according to precedence, summing up the
+ * counts (all lists at same precedence are used); if found, set cnts
+ * accordingly. */
 static void lookup(const word_t *token, wordcnts_t *cnts)
 {
     int override=0;
@@ -104,69 +94,88 @@ static void lookup(const word_t *token, wordcnts_t *cnts)
 	return;
     }
 
+    cnts->msgs_bad = cnts->msgs_good = 0;
+
     for (list=word_lists; list != NULL; list=list->next)
     {
-	size_t i;
 	dsv_t val;
+	int ret;
 
 	if (override > list->override)	/* if already found */
 	    break;
 
-	if (ds_read(list->dsh, token, &val) != 0)
-	    continue;			/* not found */
+	ret = ds_read(list->dsh, token, &val);
 
-	if (list->type == WL_IGNORE)	/* if on ignore list */
+	/* check if we have the token */
+	switch (ret) {
+	    case 0:
+		/* token found, pass */
+		break;
+	    case 1:
+		/* token not found, clear counts */
+		val.count[IX_GOOD] = 0;
+		val.count[IX_SPAM] = 0;
+		break;
+	    default:
+		abort();
+	}
+
+	if (ret == 0 && list->type == WL_IGNORE) {	/* if found on ignore list */
+	    cnts->good = cnts->bad = 0;
 	    break;
+	}
 
 	override=list->override;
 
-	for (i=0; i<COUNTOF(val.count); i++) {
-	    /* Protect against negatives */
-	    if ((int) val.count[i] < 0) {
-		val.count[i] = 0;
-		ds_write(list->dsh, token, &val);
-	    }
-	    wordprob_add(cnts, val.count[i], i);
-	}
-
-	if (DEBUG_ALGORITHM(1)) {
-	    fprintf(dbgout, "%2d %2d \n", (int) cnts->good, (int) cnts->bad);
+	if (DEBUG_ALGORITHM(2)) {
+	    fprintf(dbgout, "%6d %5u %5u %5u %5u list=%s,%c,%d ",
+		    ret, (uint)val.count[IX_GOOD], (uint)val.count[IX_SPAM],
+		    (uint)list->msgcount[IX_GOOD], (uint)list->msgcount[IX_SPAM],
+		    list->listname, list->type, list->override);
 	    word_puts(token, 0, dbgout);
 	    fputc('\n', dbgout);
 	}
+
+	cnts->good += val.count[IX_GOOD];
+	cnts->bad += val.count[IX_SPAM];
+	cnts->msgs_good += list->msgcount[IX_GOOD];
+	cnts->msgs_bad += list->msgcount[IX_SPAM];
+    }
+
+    if (DEBUG_ALGORITHM(1)) {
+	fprintf(dbgout, "%5u %5u ", (uint)cnts->bad, (uint)cnts->good);
+	word_puts(token, 0, dbgout);
+	fputc('\n', dbgout);
     }
 
     return;
 }
 
-static double msg_lookup_and_score(const word_t *token, wordcnts_t *cnts)
-{
-    double prob;
 
-    if (cnts->bad == 0 && cnts->good == 0)
+/* do wordlist lookups for the words in the wordhash
+ */
+void lookup_words(wordhash_t *wh)
+{
+    hashnode_t *node;
+
+    if (msg_count_file)	/* if mc file, already done */
+	return;
+
+    for(node = wordhash_first(wh); node != NULL; node = wordhash_next(wh))
+    {
+	word_t *token     = node->key;
+	wordprop_t *props = (wordprop_t *) node->buf;
+	wordcnts_t *cnts  = &props->cnts;
 	lookup(token, cnts);
+    }
 
-    prob = wordprob_result(cnts);
-
-    return prob;
+    return;
 }
 
-static double compute_probability(const word_t *token, wordcnts_t *cnts)
-{
-    double prob;
-
-    if (cnts->bad != 0 || cnts->good != 0 || msg_count_file)
-	/* A msg-count file already has the values needed */
-	prob = wordprob_result(cnts);
-    else
-	/* Otherwise lookup the word and get its score */
-	prob = msg_lookup_and_score(token, cnts);
-
-    return prob;
-}
 
+/** selects the best spam/non-spam indicators and calculates Robinson's S,
+ * \return -1.0 for error, S otherwise */
 double msg_compute_spamicity(wordhash_t *wh, FILE *fp) /*@globals errno@*/
-/* selects the best spam/non-spam indicators and calculates Robinson's S */
 {
     hashnode_t *node;
 
@@ -174,25 +183,18 @@ double msg_compute_spamicity(wordhash_t *wh, FILE *fp) /*@globals errno@*/
     FLOAT Q = {1.0, 0};		/* Robinson's Q */
 
     double spamicity;
+    int    err = 0;
     size_t robn = 0;
     size_t count = 0;
-    bool need_stats;
+    bool need_stats = (Rtable || passthrough || (verbose > 0)) && !fBogotune;
 
     (void) fp; 	/* quench compiler warning */
 
     if (DEBUG_ALGORITHM(2)) fprintf(dbgout, "### msg_compute_spamicity() begins\n");
 
-    Rtable |= verbose > 3;
-    need_stats = Rtable || verbose || passthrough;
-
-    if (need_stats)
-	rstats_init();
-
     if (DEBUG_ALGORITHM(2)) fprintf(dbgout, "min_dev: %f, robs: %f, robx: %f\n", 
 				    min_dev, robs, robx);
 
-    wordhash_sort(wh);
-
     for(node = wordhash_first(wh); node != NULL; node = wordhash_next(wh))
     {
 	double prob;
@@ -204,25 +206,25 @@ double msg_compute_spamicity(wordhash_t *wh, FILE *fp) /*@globals errno@*/
 	    props = (wordprop_t *) node->buf;
 	    cnts  = &props->cnts;
 	    token = node->key;
-	}
-	else {
+	} else {
 	    cnts = (wordcnts_t *) node;
 	    token = NULL;
 	}
 
 	count += 1;
 
-	prob = compute_probability(token, cnts);
+	prob = calc_prob(cnts->good, cnts->bad,
+		cnts->msgs_good, cnts->msgs_bad);
 
 	if (need_stats)
 	    rstats_add(token, prob, cnts);
 
 	/* Robinson's P and Q; accumulation step */
-        /*
+	/*
 	 * P = 1 - ((1-p1)*(1-p2)*...*(1-pn))^(1/n)     [spamminess]
-         * Q = 1 - (p1*p2*...*pn)^(1/n)                 [non-spamminess]
+	 * Q = 1 - (p1*p2*...*pn)^(1/n)                 [non-spamminess]
 	 */
-        if (fabs(EVEN_ODDS - prob) - min_dev >= EPS) {
+	if (fabs(EVEN_ODDS - prob) - min_dev >= EPS) {
 	    int e;
 
 	    P.mant *= 1-prob;
@@ -236,8 +238,8 @@ double msg_compute_spamicity(wordhash_t *wh, FILE *fp) /*@globals errno@*/
 		Q.mant = frexp(Q.mant, &e);
 		Q.exp += e;
 	    }
-            robn ++;
-        }
+	    robn ++;
+	}
 
 	if (DEBUG_ALGORITHM(3)) {
 	    (void)fprintf(dbgout, "%3lu %3lu %f ",
@@ -254,18 +256,20 @@ double msg_compute_spamicity(wordhash_t *wh, FILE *fp) /*@globals errno@*/
     spamicity = get_spamicity(robn, P, Q);
 
     if (need_stats && robn != 0)
-	rstats_fini(robn, P, Q, spamicity );
+	rstats_fini(robn, P, Q, spamicity);
 
     if (DEBUG_ALGORITHM(2)) fprintf(dbgout, "### msg_compute_spamicity() ends\n");
 
-    return (spamicity);
+    return err ? -1.0 : spamicity;
 }
 
 void score_initialize(void)
 {
-    word_t *word_robx = word_new((const byte *)ROBX_W, strlen(ROBX_W));
+    word_t *word_robx = word_news(ROBX_W);
 
-    wordlist_t *list = default_wordlist();
+    wordlist_t *list = get_default_wordlist(word_lists);
+
+    rstats_init();
 
     if (fabs(min_dev) < EPS)
 	min_dev = MIN_DEV;
@@ -278,11 +282,11 @@ void score_initialize(void)
     ** If we're registering tokens, we needn't get .MSG_COUNT
     */
 
-    compute_msg_counts();
     if (fabs(robs) < EPS)
 	robs = ROBS;
 
-    if (fabs(robx) < EPS) {
+    if (fabs(robx) < EPS)
+    {
 	/* Assign default value in case there's no wordlist
 	 * or no wordlist entry */
 	robx = ROBX;
@@ -290,12 +294,15 @@ void score_initialize(void)
 	{
 	    int ret;
 	    dsv_t val;
-	    
+
 	    /* Note: .ROBX is scaled by 1000000 in the wordlist */
 	    ret = ds_read(list->dsh, word_robx, &val);
-	    if (ret == 0) {
+	    if (ret != 0)
+		robx = ROBX;
+	    else {
+		/* If found, unscale; else use predefined value */
 		uint l_robx = val.count[IX_SPAM];
-		robx = l_robx ? (double)l_robx / 1000000 : ROBX; /* unscale */
+		robx = l_robx ? (double)l_robx / 1000000 : ROBX;
 	    }
 	}
     }
