@@ -1,14 +1,10 @@
-/* $Id: word.c,v 1.14 2004/10/08 22:28:36 m-a Exp $ */
+/* $Id: word.c,v 1.22 2005/03/14 04:14:20 relson Exp $ */
 
-/*****************************************************************************
-
-NAME:
-   word.c -- support for bogofilter's word struct
-
-AUTHOR:
-   David Relson <relson@osagesoftware.com>
-
-******************************************************************************/
+/** \file word.c
+ * support for strings of arbitrary content, implementation
+ *
+ * \author David Relson <relson@osagesoftware.com>
+ */
 
 #include "common.h"
 
@@ -17,45 +13,26 @@ AUTHOR:
 
 /* Function Definitions */
 
-word_t *word_new(const byte *text, uint leng)
+word_t  *word_news(const char *cstring)
+{
+    return word_new((const byte *)cstring, strlen(cstring));
+}
+
+word_t *word_new(const byte *text, uint len)
 {
     /* to lessen malloc/free calls, allocate struct and data in one block */
-    uint len   = (leng || !text) ? leng : (uint) strlen((const char *) text);
-    word_t *self = xmalloc(sizeof(word_t)+len+D);
+    word_t *self = xmalloc(sizeof(word_t)+len+1);
     self->leng = len;
     self->text = (byte *)((char *)self+sizeof(word_t));
-    if (text) {
+    if (text != NULL) {
 	memcpy(self->text, text, len);
-	Z(self->text[len]);			/* for easier debugging - removable */
+	self->text[len] = '\0';			/* ensure nul termination */
+    } else {
+	self->text[0] = '\0';			/* ditto for text == NULL */
     }
     return self;
 }
 
-void word_free(word_t *self)
-{
-    xfree(self);
-}
-
-word_t *word_dup(const word_t *word)
-{
-    word_t *self = xmalloc(sizeof(word_t)+word->leng+D);
-    self->leng = word->leng;
-    self->text = (byte *)((char *)self+sizeof(word_t));
-    if (word->text) {
-	memcpy(self->text, word->text, self->leng);
-	Z(self->text[self->leng]);		/* for easier debugging - removable */
-    }
-    return self;
-}
-
-word_t *word_cpy(word_t *dst, const word_t *src)
-{
-    dst->leng = src->leng;
-    memcpy(dst->text, src->text, src->leng);
-    Z(dst->text[dst->leng]);			/* for easier debugging - removable */
-    return dst;
-}
-
 int word_cmp(const word_t *w1, const word_t *w2)
 {
     uint l = min(w1->leng, w2->leng);
@@ -66,6 +43,14 @@ int word_cmp(const word_t *w1, const word_t *w2)
     return 0;
 }
 
+int word_cmps(const word_t *w, const char *s)
+{
+    word_t w2;
+    w2.leng = strlen(s);
+    w2.text = (byte *) s;
+    return word_cmp(w, &w2);
+}
+
 word_t *word_concat(const word_t *w1, const word_t *w2)
 {
     uint len = w1->leng + w2->leng;
