@@ -1,4 +1,4 @@
-/* $Id: wordlists.c,v 1.73 2004/08/07 12:31:29 relson Exp $ */
+/* $Id: wordlists.c,v 1.96 2005/04/04 11:16:22 relson Exp $ */
 
 #include "common.h"
 
@@ -6,17 +6,20 @@
 #include <ctype.h>
 #include <errno.h>
 #include <stdlib.h>
+#include <assert.h>
 
 #include "bogofilter.h"
-#include "bogohome.h"
 #include "datastore.h"
 #include "msgcounts.h"
+#include "mxcat.h"
 #include "paths.h"
 #include "rand_sleep.h"
 #include "wordlists.h"
 #include "xmalloc.h"
 #include "xstrdup.h"
 
+#include "bsdqueue.h"
+
 #define	MIN_SLEEP	0.5e+3		/* 0.5 milliseconds */
 #define	MAX_SLEEP	2.0e+6		/* 2.0 seconds */
 
@@ -26,17 +29,66 @@
 
 /* Function Definitions */
 
+#define MAX_ENVS    10
+
+/* Idea: - build dirname
+ *	 - search list of known environments if we have that dirname,
+ *	   if so, return it
+ *	 - create and insert new environment
+ *
+ * for destruction: - when closed an environment, NULL it in word_lists
+ * past the current one that match our address
+ */
+
+static LIST_HEAD(envlist, envnode) envs;
+struct envlist envlisthead;
+struct envnode {
+    LIST_ENTRY(envnode) entries;
+    void *dbe;
+    char directory[1];
+};
+
+static void *list_searchinsert(bfpath *bfp)
+{
+    uint l;
+    struct envnode *i, *n;
+
+    for (i = envlisthead.lh_first ; i ; i = i->entries.le_next) {
+	if (strcmp(bfp->dirname, &i->directory[0]) == 0)
+	    return i->dbe;
+    }
+
+    l = strlen(bfp->dirname) + 1;
+    n = xmalloc(sizeof(struct envnode) + l);
+
+    n->dbe = ds_init(bfp);
+
+    memcpy(&n->directory[0], bfp->dirname, l);
+    LIST_INSERT_HEAD(&envlisthead, n, entries);
+    return n->dbe;
+}
+
 static bool open_wordlist(wordlist_t *list, dbmode_t mode)
 {
     bool retry = false;
+    void *dbe;
+    bfpath *bfp = list->bfp;
 
     if (list->type == WL_IGNORE) 	/* open ignore list in read-only mode */
 	mode = DS_READ;
 
-    list->dsh = ds_open(bogohome, list->filepath, mode);
+    /* FIXME: create or reuse environment from filepath */
+
+    dbe = list_searchinsert(bfp);
+
+    if (dbe == NULL)
+	exit(EX_ERROR);
+
+    list->dsh = ds_open(dbe, bfp, mode); /* FIXME */
+
     if (list->dsh == NULL) {
 	int err = errno;
-	close_wordlists(true); /* unlock and close */
+	close_wordlists(false);	/* unlock and close */
 	switch(err) {
 	    /* F_SETLK can't obtain lock */
 	case EAGAIN:
@@ -52,7 +104,7 @@ static bool open_wordlist(wordlist_t *list, dbmode_t mode)
 		return false;
 	    fprintf(stderr,
 		    "Can't open file '%s' in directory '%s'.\n",
-		    list->filepath, bogohome);
+		    bfp->filename, bfp->dirname);
 	    if (err != 0)
 		fprintf(stderr,
 			"error #%d - %s.\n", err, strerror(err));
@@ -71,57 +123,101 @@ static bool open_wordlist(wordlist_t *list, dbmode_t mode)
 	} /* switch */
     } else { /* ds_open */
 	dsv_t val;
-	ds_get_msgcounts(list->dsh, &val);
-	list->msgcount[IX_GOOD] = val.goodcount;
-	list->msgcount[IX_SPAM] = val.spamcount;
-	if (wordlist_version == 0 &&
-	    ds_get_wordlist_version(list->dsh, &val))
-	    wordlist_version = val.count[0];
-    } /* ds_open */
+
+	if (ds_txn_begin(list->dsh))
+	    exit(EX_ERROR); /* XXX FIXME: do we want to retry here? */
+	switch (ds_get_msgcounts(list->dsh, &val)) {
+	    case 0:
+	    case 1:
+		list->msgcount[IX_GOOD] = val.goodcount;
+		list->msgcount[IX_SPAM] = val.spamcount;
+		if (wordlist_version == 0 &&
+			ds_get_wordlist_version(list->dsh, &val) == 0)
+		    wordlist_version = val.count[0];
+		break;
+	    case DS_ABORT_RETRY:
+		retry = true;
+		break;
+	}
+    }
+
+    /* xfree(dbe); */
 
     return retry;
 }
 
+/* Note: seems like some cleanup around here should be possible....
+** It seems wrong to call set_bogohome() from outside of paths.c.
+** (DR 04/03/05)
+*/
+static void check_wordlist_path(wordlist_t *list)
+{
+    bfpath *bfp = list->bfp;
+    bfpath_check_mode(bfp, BFP_MAY_CREATE);
+    bfpath_update(bfp);
+}
+
+/* set bogohome using first wordlist's directory
+*/
+void set_wordlist_directory(void)
+{
+    wordlist_t *list = word_lists;	/* get first wordlist */
+    if (list != NULL) {
+	bfpath *bfp = list->bfp;
+	const char *dir = get_directory_from_path(bfp->filepath);
+	if (dir != NULL) {
+	    check_wordlist_path(list);
+	    set_bogohome(dir);
+	}
+    }
+}
+
 void open_wordlists(dbmode_t mode)
 {
     bool retry = true;
 
-    if (word_lists == NULL)
-	init_wordlist("word", WORDLIST, 0, WL_REGULAR);
+    /* set bogohome using first wordlist's directory */
+    set_wordlist_directory();
 
-    ds_init();
+    LIST_INIT(&envs);
 
     while (retry) {
-	if (run_type & (REG_SPAM | REG_GOOD | UNREG_SPAM | UNREG_GOOD))
-	    retry = open_wordlist(default_wordlist(), mode);
-	else {
-	    wordlist_t *list;
-	    retry = false;
-	    for (list = word_lists; list != NULL; list = list->next) {
-		retry |= open_wordlist(list, list->type != WL_IGNORE ? mode : DS_READ);
-	    }  /* for */
+	wordlist_t *list;
+	retry = false;
+	for (list = word_lists; list != NULL ; list = list->next) {
+	    check_wordlist_path(list);
+	    retry |= open_wordlist(list, list->type == WL_IGNORE ? DS_READ : mode);
 	}
     }
 }
 
 /** close all open word lists */
-void close_wordlists(bool nosync /** Normally false, if true, do not synchronize data. This should not be used in regular operation but only to ease the disk I/O load when the lock operation failed. */)
+bool close_wordlists(bool commit /** if unset, abort */)
+    /* FIXME: we really need to look at the list's environments */
 {
+    bool err = false;
     wordlist_t *list;
+    struct envnode *i;
 
-    for ( list = word_lists; list != NULL; list = list->next )
-    {
-	if (list->dsh) ds_close(list->dsh, nosync);
-	list->dsh = NULL;
+    for (list = word_lists; list != NULL ; list = list->next) {
+	if (list->dsh) {
+	    if (commit) {
+		if (ds_txn_commit(list->dsh))
+		    err = true;
+	    } else {
+		(void)ds_txn_abort(list->dsh);
+	    }
+	    ds_close(list->dsh);
+	    list->dsh = NULL;
+	}
     }
 
-    ds_cleanup();
-}
-
-bool build_wordlist_path(char *filepath, size_t size, const char *path)
-{
-    bool ok = build_path(filepath, size, path, WORDLIST);
-    return ok;
+    while ((i = envlisthead.lh_first)) {
+	ds_cleanup(i->dbe);
+	LIST_REMOVE(i, entries);
+	xfree(i);
+    }
+    return err;
 }
 
 #ifdef COMPILE_DEAD_CODE
@@ -135,8 +231,8 @@ static void sanitycheck_lists(void)
     int listcount=0;
 
     while(1) {
-	if (!list) break;
-	if (! list->name) {
+	if (list == NULL) break;
+	if (list->name == NULL) {
 	    fprintf(stderr, "A list has no name.\n");
 	    exit(EX_ERROR);
 	}
@@ -156,29 +252,12 @@ static void sanitycheck_lists(void)
 }
 #endif
 
-/** Sum up the msgcount for each of the word lists, store the
- * count of bad messages in \a mb and the count of good messages in the
- * data bases in \a mg. */
-void compute_msg_counts(void)
-{
-    u_int32_t s = 0, g = 0;
-    wordlist_t* list;
-
-    for(list=word_lists; list != NULL; list=list->next)
-    {
-	s += list->msgcount[IX_SPAM];
-	g += list->msgcount[IX_GOOD];
-    }
-    set_msg_counts(g, s);
-}
-
 static char *spanword(char *p)
 {
     const char *delim = ", \t";
     p += strcspn(p, delim);		/* skip to end of word */
     *p++ = '\0';
-    while (isspace((unsigned char)*p)) 	/* skip trailing whitespace */
-	p += 1;
+    p += strspn(p, " \t");		/* skip trailing whitespace */
     return p;
 }
 
@@ -225,7 +304,7 @@ bool configure_wordlist(const char *val)
     tmp = spanword(tmp);
     
     init_wordlist(listname, filename, precedence, type);
-    
+
     config_setup = true;
 
     return true;
