@@ -1,4 +1,4 @@
-/* $Id: wordlists.h,v 1.13 2004/05/18 00:22:53 relson Exp $ */
+/* $Id: wordlists.h,v 1.22 2005/04/04 11:16:22 relson Exp $ */
 
 /*  constants and declarations for wordlists */
 
@@ -20,14 +20,13 @@ extern wordlist_t *word_lists;
 
 void incr_wordlist_mode(void);
 void set_wordlist_mode(const char *filepath);
-bool build_wordlist_path(char *filepath, size_t size, const char *path);
 bool configure_wordlist(const char *val);
 
-void open_wordlists(dbmode_t);
-void close_wordlists(bool);
+void open_wordlists(dbmode_t mode);
+bool close_wordlists(bool commit);
+bool query_wordlists_closed(void);
 
 void set_list_active_status(bool status);
-
-void compute_msg_counts(void);
+void set_wordlist_directory(void);
 
 #endif	/* WORDLISTS_H */
